/*
 * @(#)CWProtocol.java    2.6.1999
 */


package corewar;

/**
 * Das Interface CWProtocol definiert Konstanten fr die Kommunikation zwischen
 * den Core-War Programmen.
 *
 * @author  Ivo Kasiuk
 */
public interface CWProtocol {

    /** Identifiziert den Server. */
    byte I_AM_SERVER = 1;

    /** Identifiziert den Client. */
    byte I_AM_CLIENT = 2;
    
    /** Verbindung wird akzeptiert. */
    byte CONNECT_OK = 3;

    /** Mitteilung der Serveradresse: Es folgt ein ServerInfo .*/
    byte THIS_IS_YOUR_SERVER = 4;

    /** Mitteilung der Client-Kennnummer: Es folgt ein byte. */
    byte THIS_IS_YOUR_CLIENT = 5;

    /**
     * Der Server signalisiert dem Verteiler, da er den zugewiesenen Client
     * annehmen will.
     */
    byte READY_FOR_CLIENT = 6;

    /**
     * Mitteilung der Anzahl der vom Server bentigten Cores: Es folgt ein byte.
     */
    byte CORES_NEEDED = 7;
    
    /** Verbindung wird abgelehnt/beendet. */
    byte BYE = 8;

    /** Der Client ist bereit, die Spieldaten zu empfangen. */
    byte READY_FOR_DATA = 9;

    /** Versenden eines Cores: Es folgt ein CoreInfo. */
    byte CORE_INFO = 10;

    /** Mitteilung der Startadressen der Cores: es folgt ein int[]. */
    byte RAM_INFO = 11;

    /** Mitteilung der Spielparameter: Es folgt ein ParamInfo. */
    byte PARAM_INFO = 12;

    /**
     * Versenden der Zyklusdaten: Es folgt ein short[] fr die PCs, ein short[]
     * fr die Schreibadressen und ein byte[] fr die Schreiblngen der Cores.
     */
    byte CYCLE_INFO = 13;

    /** Signalisiert das Ende einer Runde. */
    byte END_OF_ROUND = 14;

    /** Signalisiert einen Spielabbruch. */
    byte STOP = 15;

    /**
     * Mitteilung der noch zum Spielstart fehlenden Clients: Es folgt ein byte.
     */
    byte WAIT_FOR_CORES = 16;

    /**
     * Der Server signalisiert dem Verteiler, da er den zugewiesenen Client
     * nicht annehmen will.
     */
    byte DONT_NEED = 17;

    /** Signalisiert eine fehlerhafte Verbindung. */
    byte CONNECTION_ERROR = -1;

    /** Standardport fr die Anmeldung von Clients beim Verteiler. */
    int DEFAULT_META_CLIENT_PORT = 7530;

    /** Standardport fr die Anmeldung von Servern beim Verteiler. */
    int DEFAULT_META_SERVER_PORT = 7630;

    /** Standardport fr die Anmeldung von Clients beim Server. */
    int DEFAULT_SERVER_PORT = 6330;

    /**
     * Anzahl von Ports, die der Verteiler durchprobiert, wenn der Standardport
     * nicht zur Verfgung steht.
     */
    int MAX_META_PORTS = 10;

    /**
     * Anzahl von Ports, die der Server durchprobiert, wenn der Standardport
     * nicht zur Verfgung steht.
     */
    int MAX_SERVER_PORTS = 20;
    
    /** Zeitgrenze whrend der Anmeldung in ms.*/
    int CONNECT_TIMEOUT = 3000;

    /**
     * Zeitgrenze vom Senden von READY_FOR_CLIENT durch den Server bis zur
     * Anmeldung des Clients.
     */
    int SERVER_CLIENT_TIMEOUT = 10000;
}

