/*
 * @(#)MessageBox.java    15.6.1999
 */


package corewar;

import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Panel;
import java.awt.Button;
import java.awt.Label;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.StringTokenizer;

/**
 * Die Klasse MessageBox stellt eine einfache, modale Dialogbox zur Verfgung.
 *
 * @author  Ivo Kasiuk
 */
public class MessageBox extends Dialog {

    /** Bezeichnet den "Ok"-Button. */
    public static final int OK = 1 << 0;

    /** Bezeichnet den "Ja"-Button. */
    public static final int YES = 1 << 1;

    /** Bezeichnet den "Nein"-Button. */
    public static final int NO = 1 << 2;

    /** Bezeichnet den "Abbrechen"-Button. */
    public static final int CANCEL = 1 << 3;

    /* Die Titel der Buttons. buttonTitles[n] enspricht dem Flag 1<<n. */
    private static String[] buttonTitles = { "   Ok   ", "   Ja   ", "  Nein  ",
                                             "Abbrechen" };

    /* Der angewhlte Button (OK, YES,...). */
    private int result;
    
    /**
     * Initialisiert die Dialogbox.
     * @param message  Die Nachricht, die in der Dialogbox erscheint. Dieser
     *                 Text darf Zeilenumbrche enthalten.
     * @param title    Der Titel der Dialogbox.
     * @param parent   Das Fenster, von dem die Dialogbox erzeigt wird.
     * @param buttons  Eine Kombination der Flags (OK, YES,...), die angibt,
     *                 welche Buttons die Dialogbox haben soll.
     * @exception IllegalArgumentException  wennn parent den Wert null hat.
     */
    public MessageBox(String message, String title, Frame parent, int buttons)
            throws IllegalArgumentException {
        super(parent, title, true);
        setLayout(new GridLayout(0, 1));

        /* Den Text zeilenweise hinzufgen. */
        StringTokenizer tokenizer = new StringTokenizer(message, "\n");
        while (tokenizer.hasMoreTokens()) {
            add(new Label(tokenizer.nextToken(), Label.CENTER));
        }
        
        ActionListener buttonListener = new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                /* Die gemerkte Button-Nummer (s.u.) auslesen. */
                result = 1 << ((int) (event.getActionCommand().charAt(0)));
                setVisible(false);
            }
        };

        /* Buttons so hinzufgen, da sie alle die gleiche Gre haben. */
        Panel bottomPanel = new Panel();
        Panel buttonPanel = new Panel(new GridLayout(1, 0, 5, 0));
        for (int i=0; i<buttonTitles.length; i++) {
            if ((buttons & (1 << i)) != 0) {
                Button button = new Button(buttonTitles[i]);
                /* Nummer des Buttons (OK=0, YES=1,...) merken. */
                button.setActionCommand(String.valueOf((char) i));
                button.addActionListener(buttonListener);
                buttonPanel.add(button);
            }
        }
        bottomPanel.add(buttonPanel);
        add(bottomPanel);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent event) {
                result = CANCEL;
                setVisible(false);
            }
        });

        pack();
        setResizable(false);
    }
   
    /**
     * Zeigt den Dialog an und wartet, bis er beendet wird.
     * @return  Der angewhlte Button (OK, YES,...).
     */
    public int execute() {
        show();
        return result;
    }

    /*
    public static void main(String[] args) throws IllegalArgumentException {
        MessageBox mb = new MessageBox("Das ist die Nachricht. "
                                    + "Die erste Zeile ist ziemlich lang.\n"
                                       + "Das ist die zweite Zeile.",
                                       "Test", new Frame(),
                                       MessageBox.YES | MessageBox.NO
                                       | MessageBox.CANCEL);
        System.out.println(mb.execute());
        System.exit(0);
    }
    */
}
