/*
 * @(#)NetworkSSocket.java    18.6.1999
 */


package corewar;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * Die Klasse NetworkSSocket ist eine abstrakte Oberklasse, die es ermglicht,
 * an einem ServerSocket mit einem eigenen Thread stndig auf Clients zu warten.
 * In einer abgeleiteten Klasse mu man handleConnection(Socket) berschreiben,
 * um auf einen neuen Client zu reagieren.
 */
public abstract class NetworkSSocket extends ServerSocket
        implements Runnable, CWProtocol {

    private boolean locked = true;  /* true, wenn keine Clients angenommen
                                       werden sollen */
    private boolean closed = false; /* true, wenn close() aufgerufen wurde */
    
    /**
     * Socket einrichten.
     * @param port    Port, an dem der Socket eingerichtet werden soll.
     * @param locked  true, wenn der Socket gleich gesperrt werden soll.
     * @exception IOException  wenn das Einrichten des Sockets fehlschlgt.
     */
    public NetworkSSocket(int port, boolean locked) throws IOException {
        super(port);
        this.locked = locked;
    }

    /**
     * Stndig auf Clients warten. Wenn der Socket gesperrt ist werden alle
     * Clients sofort abgelehnt, sonst wird handleClient(Socket) aufgerufen.
     */
    public void run() {
        while (!closed) {
            try {
                Socket client = accept();
                if (locked) {
                    client.getOutputStream().write(BYE);
                } else {
                    handleConnection(client);
                }
            } catch (IOException ioe) {
                if (!closed) {
                    System.err.println("NetworkSSocket: Fehler beim Verbinden");
                }
            }
        }
    }

    /**
     * Den Thread zum Warten auf Clients starten.
     */
    public void start() {
        (new Thread(this)).start();
    }

    /**
     * Auf einen neuen Client reagieren.
     * @param client  Der Socket, der die Verbindung zu dem neuen Client
     *                darstellt.
     */
    protected abstract void handleConnection(Socket client);

    /**
     * Den Socket fr Clients sperren oder freigeben.
     * @param locked  true, wenn der Socket gesperrt werden soll. 
     */
    public void setLocked(boolean locked) {
        this.locked = locked;
    }

    /**
     * Feststellen, ob der Socket fr Clients gesperrt oder feigegeben ist.
     * @return  true, wenn der Socket gesperrt ist.
     */
    public boolean getLocked() {
        return locked;
    }

    /**
     * Den Socket schlieen. Der Thread zum Warten auf Clients wird gestoppt.
     * @exception IOException  wenn beim Schlieen ein Fehler auftritt.
     */
    public void close() throws IOException {
        closed = true;
        super.close();
    }   
}
