/*
 * @(#)ParamDisplay.java    14.6.1999
 */


package corewar;

import java.awt.Panel;
import java.awt.Label;
import java.awt.GridLayout;
import java.awt.Component;
import java.util.Vector;

  import java.awt.Frame;
  import java.awt.event.WindowAdapter;
  import java.awt.event.WindowEvent;
  import java.awt.event.ItemListener;


/**
 * Die Klasse ParamDisplay ist ein von Panel abgeleitetes Bildschirmelement, das die in einem 
 * ParamInfo-Objekt enthaltenen Infomationen anzeigt, wozu es Labels mit den Informationen und
 * Beschreibungen dazu verwendet.
 *
 * @author  Carsten Buschmann
 */
public class ParamDisplay extends Panel {

    /* Diese Konstanten bezeichnen die dargestellten Informationen und 
       entsprechen dem Index der jeweiligen Information in den Vector-Objekten */

   public static final int INFO_TOTAL_ROUNDS = 0;    // Anzahl der Runden pro Spiel
   public static final int INFO_MAX_CYCLES = 1;      // Anzahl der Zyklen pro Spielrunde
   public static final int INFO_RAM_SIZE = 2;        // Gr"o"se des Speichers
   public static final int INFO_MAX_CORE_SIZE = 3;   // maximale Gr"o"se der Cores
   public static final int INFO_MAX_CORE_NUMBER = 4; // maximale Anzahl der Cores
   public static final int INFO_MIN_DISTANCE = 5;    // minimaler Abstand zwischen den Cores im Speicher

   public static final int MAX_PARAM_INDEX = 5;    // Grter von ParamDisplay verwendeter Vector-Index


   /** 
    *  Diese zwei Vector-Objekte enthalten Label-Objekte.
    *  description enth"alt die Beschreibungen der Informationen und
    *  information die dazugeh"origen Informationen. 
    */
      
   protected Vector description;
   protected Vector information;

   /**
     * In diesem ParamInfo-Objekt speichert ParamInfo seine Informationen.
     */
     
   protected ParamInfo params;
   
   /**
     * Erstellt ein neues ParamDisplay und stellt zunchst nur die Beschreibungen,
     * aber keine Informationen dar.
     */
     
   public ParamDisplay() {
   	   super(new GridLayout(0, 2));
   	   description = new Vector();
   	   description.addElement(new Label(" Anzahl Runden:"));
   	   description.addElement(new Label(" Zyklen/Runde:"));
   	   description.addElement(new Label(" Speichergre:"));
   	   description.addElement(new Label(" max. Coresgre:"));
   	   description.addElement(new Label(" Coreanzahl:"));
   	   description.addElement(new Label(" min. Coreabstand:"));
   
       for (byte b=0; b<=MAX_PARAM_INDEX; b++) add((Component)description.elementAt(b), b);

   	   information = new Vector();
   	   information.addElement(new Label(""));
   	   information.addElement(new Label(""));
       information.addElement(new Label(""));
       information.addElement(new Label(""));
       information.addElement(new Label(""));
       information.addElement(new Label(""));

       for (byte b=0; b<=MAX_PARAM_INDEX; b++) add((Component)information.elementAt(b),2*b+1);
   }
   
   
   /**
     * Erstellt ein neues ParamDisplay mit den in param angegebenen Parametern, die intern
     * mittels setParam( param ) gestzt werden. 
     * @param param  Die anzuzeigenden Parameter     */
     
   public ParamDisplay(ParamInfo param) {
   	   this();
   	   setParamInfo(param);
   }

   /**
     * Liefert die gesetzten Parameter zurck. 
     * @return die aktuell gesetzten Parameter in einer ParamInfo    */
     
   public ParamInfo getParamInfo() {
   	 return params;
   }

   /**
     * Stellt das ParamDisplay neu dar. */
     
   public void refresh() {
       validate();
   }

   /**
     * Setzt die in param angegebenen Parameter und fgt die Labels mit den bergebenen
     * Informationen ein.
     * @param param  Die anzuzeigenden Parameter     */

   public void setParamInfo(ParamInfo param) {
       params = param;
       if (param == null) {
   	   ((Label)information.elementAt(0)).setText("");
   	   ((Label)information.elementAt(1)).setText("");
   	   ((Label)information.elementAt(2)).setText("");
   	   ((Label)information.elementAt(3)).setText("");
   	   ((Label)information.elementAt(4)).setText("");
   	   ((Label)information.elementAt(5)).setText("");
       } else {
   	   ((Label)information.elementAt(0)).setText(Byte.toString(param.getTotalRounds()));
   	   ((Label)information.elementAt(1)).setText(Integer.toString(param.getMaxCycles()));
   	   ((Label)information.elementAt(2)).setText(Integer.toString(param.getRAMSize()));
   	   ((Label)information.elementAt(3)).setText(Short.toString(param.getMaxCoreSize()));
   	   ((Label)information.elementAt(4)).setText(Byte.toString(param.getCoreNumber()));
   	   ((Label)information.elementAt(5)).setText(Short.toString(param.getMinDistance()));
       }
    }

   /**
     * Macht einzelne Zeilen im Paramdisplay (un)sichtbar
     * @param information  Konstante derjenigen Zeile, die verstellt werden soll 
     * @param visible      besagt, ob das Element angezeigt werden soll oder nicht  */

   public void setVisible(int information, boolean visible) throws IllegalArgumentException {
       try {
         Component inf = (Component)this.information.elementAt(information);
         Component des = (Component)description.elementAt(information);
         
         if (inf.isVisible() != visible) {
       	   inf.setVisible(visible);
         	 des.setVisible(visible);
         	 if (visible) {
         	 	 int pos = information;
         	 	 for (int j = information-1; j >= 0; j--) 
       	   	 	 if (!((Component)this.information.elementAt(j)).isVisible() )
       	 	   	 	 pos -= 2;
         	 	 add(inf, pos);
         	 	 add(des, pos + 1);
         	 } else {
         	 	 remove(inf);
       	   	 remove(des);
         	 }
       	 }
       }
       catch(Exception e) {throw new IllegalArgumentException();}
   }
       	 	 	 
    
      public static void main(String[] args) {
          ParamInfo p = new ParamInfo();
          p.setTotalRounds( (byte)10 );
          p.setMaxCycles( 10000 );
          p.setRAMSize( 32 * 1024);
          p.setMaxCoreSize( (short)256 );
          p.setCoreNumber( (byte)5 );
          p.setMinDistance( (short)256 );
          
          ParamDisplay d = new ParamDisplay(p);
          d.setParamInfo(p);
          Frame f = new Frame("Test ParamDisplay");
          f.add(d);
          
          f.addWindowListener(new WindowAdapter() {
              public void windowClosing(WindowEvent event) {
                  System.exit(0);
              }
          });
          f.pack();
          f.setVisible(true);
      }
    

}