/*
 * @(#)SatusDisplay.java    14.6.1999
 */


package corewar;

import java.awt.LayoutManager;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Component;
import java.util.Vector;

  import java.awt.Frame;
  import java.awt.event.WindowAdapter;
  import java.awt.event.WindowEvent;
  import java.awt.event.ItemListener;


/**
 * Die Klasse StatusDisplay ist ein von Panel abgeleitetes Bildschirmelement, das die in einem 
 * GameInfo-Objekt enthaltenen Infomationen anzeigt, wozu es Labels mit den Informationen und
 * Beschreibungen verwendet.
 *
 * @author  Carsten Buschmann
 */
public class StatusDisplay extends ParamDisplay {

    /* Diese Konstanten bezeichnen die dargestellten Informationen und 
       entsprechen dem Index der jeweiligen Information in den Vector-Objekten */

   public static final int INFO_ROUND_NUMBER = 6;    // aktuelle Runde
   public static final int INFO_CYCLE_NUMBER = 7;    // aktueller Zyklus dieser Runde
   public static final int INFO_ACTIVE_CORES = 8;    // Anz. noch aktiver Cores
   public static final int INFO_SERVER_NAME = 9;     // Name des Servers
   
   public static final int MAX_STATUS_INDEX = 9;    // Grter von StatusDisplay verwendeter Vector-Index


   //protected GameInfo gameInfo;
   
   /**
     * Erstellt ein neues StatusDisplay und stellt die in gameInfo bergebenen
     * Informationen dar.
     */
    
   public StatusDisplay(GameInfo gameInfo) {
       this();
       setGameInfo( gameInfo );       
   }
   
   /**
     * Erstellt ein neues StatusDisplay und stellt zunchst nur die Beschreibungen,
     * aber keine Informationen dar.
     */
   public StatusDisplay() {
   	   super();
   	   description.addElement(new Label(" Akt. Runde:"));
   	   description.addElement(new Label(" Akt. Zyklus:"));
   	   description.addElement(new Label(" Aktive Cores:"));
   	   description.addElement(new Label(" Verteiler:"));
   	   
       for (int b=MAX_PARAM_INDEX+1; b<=MAX_STATUS_INDEX; b++) add((Component)description.elementAt(b), MAX_PARAM_INDEX+1+b);

       information.addElement(new Label(""));
			 information.addElement(new Label(""));
       information.addElement(new Label(""));
       information.addElement(new Label(""));

       for (int b=MAX_PARAM_INDEX+1; b<=MAX_STATUS_INDEX; b++) add((Component)information.elementAt(b),2*b+1);
   }

   /** Liefert die dargestellten Informationen zurck
     * @return Ein GameInfo-Objekt mit den Informationen */

   public GameInfo getGameInfo() {
   	   return ((GameInfo)params);
   }
      
   /**
     * Incrementiert die Zyklenzahl und stellt diese dar.*/

   public void incrementCycle() {
   	   setCycle(((GameInfo)params).getCycleNumber() + 1);
   }

   /**
     * Incrementiert die Rundenzahl und stellt diese dar.*/

   public void incrementRound() {
   	   setRound((byte)(((GameInfo)params).getRoundNumber() + 1));
   }
   
   /**
     * Setzt die in activeCores bergebene Coreanzahl und stellt diese dar.
     * @param activeCores  Die anzuzeigende Coreanzahl     */

   public void setActiveCores(byte activeCores) {
   	   ((GameInfo)params).setActiveCores(activeCores);
   	   Label l = (Label)information.elementAt(INFO_ACTIVE_CORES);
       l.setText(Byte.toString(activeCores));
   }
   
   /**
     * Setzt die in cycle bergebene Zyklenzahl und stellt diese dar.
     * @param cycle  Die anzuzeigende Zyklenzahl     */

   public void setCycle(int cycle) {
   	   ((GameInfo)params).setCycleNumber(cycle);
   	   Label l = (Label)information.elementAt(INFO_CYCLE_NUMBER);
       l.setText(Integer.toString(cycle));
   }

   /**
     * Setzt die im GameInfo-Object param bergebenen Informationen und stellt diese dar.
     * @param status Die anzuzeigende ParamInfo */

   public void setGameInfo(GameInfo status) {
       super.setParamInfo(status);
       
   	   ((Label)information.elementAt(6)).setText(Byte.toString(((GameInfo)params).getRoundNumber()));
   	   ((Label)information.elementAt(7)).setText(Integer.toString(((GameInfo)params).getCycleNumber()));
   	   ((Label)information.elementAt(8)).setText(Byte.toString(((GameInfo)params).getActiveCores()));
   	   ((Label)information.elementAt(9)).setText(((GameInfo)params).getServerName());
   }

   /**
     * Setzt die im ParamInfo-Object param Informationen innerhalb des internen
     * GameInfo-Objekt und stellt diese dar.
     * @param param  Die anzuzeigende ParamInfo */

    public void setParamInfo(ParamInfo param) {
      	             	 

      	GameInfo gi = getGameInfo();
      	if ( gi == null ) { gi = new GameInfo();  }
      	gi.setTotalRounds( param.getTotalRounds());
      	gi.setMaxCycles( param.getMaxCycles());
    	  gi.setRAMSize( param.getRAMSize());
      	gi.setMaxCoreSize( param.getMaxCoreSize());
      	gi.setCoreNumber( param.getCoreNumber());
      	gi.setMinDistance( param.getMinDistance());
    	  setGameInfo(gi);

    }

   /**
     * Setzt die in round bergebene Rundenzahl und stellt diese dar.
     * @param round  Die anzuzeigende Rundenzahl     */

   public void setRound(byte round) {
   	   ((GameInfo)params).setRoundNumber(round);
   	   Label l = (Label)information.elementAt(INFO_ROUND_NUMBER);
       l.setText(Byte.toString(round));
   }

   /**
     * Setzt den in server bergebenen Servernamen und stellt diesen dar.
     * @param server  Der anzuzeigende Servernamen     */

   public void setServer(String server) {
   	   ((GameInfo)params).setServerName(server);
   	   Label l = (Label)information.elementAt(INFO_SERVER_NAME);
       l.setText(server);
   }
   
      
      public static void main(String[] args) {
          GameInfo p = new GameInfo();
          p.setTotalRounds( (byte)10 );
          p.setMaxCycles( 10000 );
          p.setRAMSize( 32 * 1024);
          p.setMaxCoreSize( (short)256 );
          p.setCoreNumber( (byte)5 );
          p.setMinDistance( (short)256 );
          
          p.setRoundNumber( (byte)7 );
          p.setCycleNumber( (int)7586 );
          p.setActiveCores( (byte)3 );
          p.setServerName( "rzcipns1.rz.tu-bs.de" );
          
          StatusDisplay d = new StatusDisplay(p);
          Frame f = new Frame("Test StatusDisplay");
          f.add(d);
          
          f.addWindowListener(new WindowAdapter() {
              public void windowClosing(WindowEvent event) {
                  System.exit(0);
              }
          });
          f.pack();
          f.setVisible(true);
          
          for ( int i= 1; i<1000; i++) d.setCycle( i );
      }
    

}
