/*
 * @(#)ClientCoreList.java    7.6.1999
 */


package corewar.client;

import corewar.ExtendedList;
import corewar.client.CoreStatus;

import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * Die Klasse ClientCoreList verwaltet eine Liste von CoreStatus-Objekten und
 * stellt sie visuell dar.
 *
 * @author  Ivo Kasiuk
 */
public class ClientCoreList extends ExtendedList {

    /* Die Spaltentitel von links nach rechts. */
    private final static String[] titles = { "Core", "Spieler", "Gre",
                                             "Status", "Punkte"  };

    /* Die Spaltenbreiten von links nach rechts. */
    private final static int[] columnWidths = { 20, 20, 5, 6, 6 };
    
    /**
     * Erstellt eine neue Liste.
     */
    public ClientCoreList(int lines) {
        super(lines);
    }

    /**
     * Liefert die Spaltenbreiten der Tabelle.
     * @return  Ein Array von Spaltenbreiten int Zeichen.
     */
    protected int[] getColumnWidths() {
        return columnWidths;
    }

    /**
     * Liefert die berschriften der Spalten.
     * @return  Ein Array von Spaltenberschriften.
     */
    protected String[] getTitles() {
        return titles;
    }

    /**
     * Liefert die Eintrge einer Zeile.
     * @param element  Das CoreStatus-Objekt, zu dem diese Zeile gehrt.
     */
    protected String[] getEntries(Object element) {
        String[]   result = new String[5];
        CoreStatus status = (CoreStatus) element;
        result[0] = status.coreName;
        result[1] = status.author;
        result[2] = String.valueOf(status.size);
        result[3] = status.alive ? "Aktiv" : "Tot";
        result[4] = String.valueOf(status.score);
        return result;
    }

    /**
     * ndert den Punktestand eines Cores.
     * @param coreNumber  Der Index des betroffenen Cores in der Liste.
     * @param score       Der neue Punktestand.
     * @exception IndexOutOfBoundsException  wenn coreNumber einen ungltigen
     *                Wert hat.
     */
    public void setScore(int coreNumber, byte score)
            throws IndexOutOfBoundsException {
        ((CoreStatus) getElements().elementAt(coreNumber)).score = score;
        refreshLine(coreNumber);
    }

    /**
     * ndert den Punktestand eines Cores relativ zum alten Punktstand.
     * @param coreNumber  Der Index des betroffenen Cores in der Liste.
     * @param score       Die Anzahl der zum Punktstand zu addierenden Punkte.
     * @exception IndexOutOfBoundsException  wenn coreNumber einen ungltigen
     *                Wert hat.
     */
    public void addToScore(int coreNumber, byte points)
            throws IndexOutOfBoundsException {
        ((CoreStatus) getElements().elementAt(coreNumber)).score += points;
        refreshLine(coreNumber);
    }

    /**
     * ndert den Status eines Cores.
     * @param coreNumber  Der Index des betroffenen Cores in der Liste.
     * @param alive       Der neue Status (true=aktiv).
     * @exception IndexOutOfBoundsException  wenn coreNumber einen ungltigen
     *                Wert hat.
     */
    public void setState(int coreNumber, boolean alive)
            throws IndexOutOfBoundsException {
        ((CoreStatus) getElements().elementAt(coreNumber)).alive = alive;
        refreshLine(coreNumber);
    }

    public static void main(String[] args) {
        ClientCoreList list = new ClientCoreList(10);
        CoreStatus     cs   = new CoreStatus();
        cs.coreName = "Super Core";
        cs.author   = "Coremaster";
        cs.size     = 364;
        cs.alive    = true;
        cs.score    = 2;
        list.addElement(cs);
        cs = new CoreStatus();
        cs.coreName = "Raupe";
        cs.author   = "Anonymous";
        cs.size     = 6;
        cs.alive    = false;
        cs.score    = 4;
        list.addElement(cs);
        cs = new CoreStatus();
        cs.coreName = "Dabeiseinisalles";
        cs.author   = "Beginner";
        cs.size     = 52;
        cs.alive    = false;
        cs.score    = 1;
        list.addElement(cs);
        Frame f = new Frame("Test");
        f.add(list, "Center");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent event) {
                System.exit(0);
            }
        });
        f.pack();
        f.setVisible(true);
        list.setState(1, true);
        list.setScore(2, (byte) 0);
        list.addToScore(0, (byte) 1);
    }
}
