/*
 * @(#)ClientMetaSocket.java    19.6.1999
 */


package corewar.client;

import corewar.NetworkSocket;
import corewar.ServerInfo;
import corewar.CoreInfo;
import java.net.InetAddress;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;

/**
 * Die Klasse MetaClientSocket stellt eine Netzwerverbindung vom Client zum
 * Verteiler dar. Sie ist fr die gesamte Kommunikation des Clients mit dem
 * Verteiler verantwortlich.
 *
 * @author  Ivo Kasiuk
 */
public class ClientMetaSocket extends NetworkSocket {

    /* Die Identifikation des Clients fr die Netzwerkkommunikation. */
    private static final byte[] clientID = { I_AM_CLIENT };

    /* Das empfangene ServerInfo-Objekt. */
    private ServerInfo serverInfo = null;

    /* Fehlermeldung, falls ein Fehler aufgetreten ist. */
    private String error = null;
    
    /**
     * Baut eine Netzwerkverbindung zu einem Verteiler auf und sendet ihm einen
     * Core (der Objektcode wird nicht mitgesendet).
     * @param hostName Der Hostname des Verteilers.
     * @param coreInfo Das CoreInfo-Objekt, das dem Verteiler gesendet wird.
     * @exception IOException  wenn bei der Anmeldung ein Fehler auftritt.
     */
    public ClientMetaSocket(String hostName, CoreInfo coreInfo)
            throws IOException {
        super(hostName, DEFAULT_META_CLIENT_PORT, clientID);
        ObjectOutputStream out   = getOutStream();
        byte[]             core  = coreInfo.core;
        int[]              reloc = coreInfo.reloc;
        coreInfo.core  = null;
        coreInfo.reloc = null;
        out.writeByte(CORE_INFO);
        out.writeObject(coreInfo);
        out.flush();
        coreInfo.core  = core;
        coreInfo.reloc = reloc;
    }

    /**
     * Wartet, bis der Verteiler diesem Client einen Server zuweist.
     * @return  Ein ServerInfo-Objekt, das den zugewiesenen Server bezeichnet.
     * @exception IOException  wenn whrend des Wartens ein Fehler auftritt.
     */
    public ServerInfo waitForServer() throws IOException {
        run();
        if (error != null) {
            throw new IOException(error);
        }
        return serverInfo;
    }

    /**
     * Reagiert auf die vom Verteiler gesendeten Nachrichten.
     * @param msgID  Die Identifikationsnummer der Nachricht.
     * @exception IOException  wenn bei der Kommunikation ein Fehler auftritt.
     */
    protected boolean handleMessage(byte msgID) throws IOException {
        switch (msgID) {
        case THIS_IS_YOUR_SERVER:
            serverInfo = null;
            try {
                serverInfo = (ServerInfo) getInStream().readObject();
            } catch (Exception e) {}
            if (serverInfo == null) {
                throw new IOException("Fehler in der Netzwerkverbindung");
            }
            getSocket().close();
            return false;
        case BYE:
            error = "Verbindung vom Verteiler abgebrochen";
            return super.handleMessage(msgID);
        case CONNECTION_ERROR:
            error = "Fehler in der Netzwerkverbindung";
            /* kein break */
        default:
            return super.handleMessage(msgID);
        }
    }
}

