/*
 * @(#)ClientProperties.java    17.6.1999
 */


package corewar.client;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

/**
 * Die Klasse ClientProperties ist ein abstraktes Datenobjekt zur Verwaltung der
 * permanent gespeicherten Einstellungen des Client. 
 *
 * @author  Ivo Kasiuk
 */
public class ClientProperties  {

    /* Der Dateiname fr die Properties-Datei. */
    private static final String FILE_NAME = "CoreWarClient.properties";
    
    /* Das Properties-Objekt mit den Einstellungen fr den Client. */
    private static Properties properties = new Properties();
    
    /**
     * Liefert das Properties-Objekt mit den Einstellungen fr den Client.
     */
    public static Properties getProperties() {
        return properties;
    }

    /**
     * Ldt die Einstellungen.
     * @return  Das Properties-Objekt mit den Einstellungen fr den Client.
     * @exception IOException  wenn beim Laden ein Fehler auftritt.
     */
    public static Properties load() throws IOException {
        properties.load(new FileInputStream(FILE_NAME));
        return properties;
    }

    /**
     * Speichert die Einstellungen.
     * @exception IOException  wenn beim Speichern ein Fehler auftritt.
     */
    public static void save() throws IOException {
        properties.save(new FileOutputStream(FILE_NAME),
                        "Einstellungen fr den CoreWar-Client");
    }

    public static void main(String[] args) throws IOException {
        load();
        getProperties().list(System.out);
    }
}
