/*
 * @(#)CoreDialog.java   07.07.1999
 */
 
package corewar.client;

import corewar.CoreInfo;
import corewar.compiler.CompilerException;
import corewar.compiler.Compiler;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.util.Properties;

/**
 * Die Klasse erzeugt eine Dialogbox zur Auswahl und Compilierung eines Cores.
 *
 * @author Hendrick Kay
 */
public class CoreDialog extends Frame {

    /** @serial Textfeld fr Quelltextdatei */
    private TextField coreFilename = new TextField("");

    /** @serial Info-Feld fr Corenamen */
    private Label coreName = new Label("");

    /** @serial Info-Feld fr Coregre */
    private Label coreSize = new Label("");

    /** @serial Info-Feld fr Coreautor */
    private Label coreAuthor = new Label("");

    /** @serial Feld fr Statusmeldungen */
    private TextArea coreDialogStatus =
        new TextArea("", 0, 0, TextArea.SCROLLBARS_VERTICAL_ONLY);

    /** @serial Kompilieren-Button */
    private Button compile = new Button("Kompilieren");

    /** @serial Abbrechen-Button */
    private Button cancel = new Button("Zurck");

    /** @serial Core-Laden-Button*/
    private Button loadCoreButton = new Button("Core ffnen");

    /** @serial Pfad der Quelltextdatei */
    private String path = null;

    /** @serial Name der Quelltextdatei */
    private String filename = null;

    /** @serial compilierter Core */
    private CoreInfo coreInfo = null;

    /** @serial File-Objekt auf den Quelltext */
    private File sourceFile = null;

    private static final String PROPERTY_COREFILE = "coreSourceFile";

    /**
     * Erzeugt den Dialog dur Coreauswahl 
     * 
     */
    public CoreDialog() {

        super("CoreDialog");

        // Unter Windows die Hintergrundfarbe korrigieren
        if (System.getProperty("os.name").startsWith("Windows")) {
            setBackground(SystemColor.control);
        }

        coreFilename.setEditable(false);


        Properties properties = ClientProperties.getProperties();

        // Property lesen
        filename = properties.getProperty(PROPERTY_COREFILE, "");

        // Kompilier-Button richtig setzen
        if ( filename == "") {
            compile.setEnabled(false);
        }
        else {

            // Datei-Objekt erzeugen
            sourceFile = new File(filename);

            // prfen, ob Datei existiert
            if ( sourceFile.exists() ) {

                // Button zum kompilieren soll nun anwhlbar sein
                compile.setEnabled(true);

                // TextField auf das Ergebnis vom Datei-Dialog setzen
                coreFilename.setText(filename);
                // Statusmeldung ausgeben
                coreDialogStatus.append("Datei " + filename + " gewhlt.\n");
            }
            else {
                // Button zum kompilieren soll nicht anwhlbar sein
                compile.setEnabled(false);
            }

        }

        // 3 Panels fr diverse Buttons, Labels,...
        Panel infoPanel = new Panel( new GridLayout(3,6) );
        Panel fileselectionPanel = new Panel( new BorderLayout() );
        Panel controlPanel = new Panel( new FlowLayout() );

        // Label dem infoPanel hinzufgen
        infoPanel.add( new Label("Corename:    ", Label.RIGHT) );
        infoPanel.add(coreName);
        infoPanel.add( new Label("Coregre:    ", Label.RIGHT) );
        infoPanel.add(coreSize);
        infoPanel.add( new Label("Autor:    ", Label.RIGHT) );
        infoPanel.add(coreAuthor);

        // Elemente in das fileselectionPanel einfgen
        fileselectionPanel.add(loadCoreButton,"West");
        fileselectionPanel.add(coreFilename,"Center");
		fileselectionPanel.add(infoPanel,"South");

        // Elemente in das controlPanel einfgen
        controlPanel.add(compile);
        controlPanel.add(cancel);

        // ActionListener einrichten
        loadCoreButton.addActionListener( new LoadCoreListener() );
        compile.addActionListener( new compileListener() );
        cancel.addActionListener( new cancelListener() );

        // Statusfeld soll nicht editierbar sein
        coreDialogStatus.setEditable(false);

        // alle Panel und das Statusfeld im Frame einfgen
        add(fileselectionPanel,"North");
        add(coreDialogStatus,"Center");
        add(controlPanel,"South");

        pack();
        centerScreen();
    }

    private void centerScreen() {
        Dimension dim = getToolkit().getScreenSize();
        Rectangle abounds = getBounds();
        setLocation((dim.width - abounds.width) / 2,
                    (dim.height - abounds.height) / 2);
    }

    class LoadCoreListener implements ActionListener {

        public void actionPerformed(ActionEvent ae) {

            // Dateiauswahl-Dialog erzeugen
            FileDialog fileDialog = new FileDialog(CoreDialog.this,
                                                   "Core-Quelltext whlen",
                                                   FileDialog.LOAD );
            // Datei-Dialog anzeigen
	    fileDialog.show();

            // gewhlten Dateinamen ermitteln
	    filename = fileDialog.getFile();
            path     = fileDialog.getDirectory();

            // wurde "CANCEL" geklickt?
            if ( filename != null ) {

                // Property speichern
                Properties properties = ClientProperties.getProperties();
                properties.put(PROPERTY_COREFILE, filename);

                // Datei-Objekt erzeugen
                sourceFile = new File(path, filename);
                // prfen, ob Datei existiert
                if ( sourceFile.exists() ) {
                    // Button zum kompilieren soll nun anwhlbar sein
                    compile.setEnabled(true);

                    // TextField auf das Ergebnis vom Datei-Dialog setzen
                    coreFilename.setText(filename);
                    // Statusmeldung ausgeben
                    coreDialogStatus.append("Datei " + filename + " gewhlt.\n");
                }
                else {
                    // Button zum kompilieren soll nicht anwhlbar sein
                    compile.setEnabled(false);
                }
            }
            else {
                coreDialogStatus.append("Dateiauswahl abgebrochen\n");
            }
        }

    } // LoadCoreListener

    class compileListener implements ActionListener {

        public void actionPerformed(ActionEvent ae) {

            try {
                CoreDialog.this.compile.setEnabled(false);
                CoreDialog.this.cancel.setEnabled(false);
                CoreDialog.this.loadCoreButton.setEnabled(false);
                CoreDialog.this.coreFilename.setEnabled(false);
                coreDialogStatus.append("Core wird kompiliert...\n");
                Compiler compiler = new Compiler( sourceFile );
                coreInfo = compiler.compile();
                coreDialogStatus.append("Kompilierung erfolgreich!\n");
                coreName.setText(coreInfo.coreName);
                coreSize.setText( new Short(coreInfo.size).toString() );
                coreAuthor.setText(coreInfo.author);
            }
            catch (IOException e) {
                coreDialogStatus.append( e.getMessage() );
                coreName.setText("");
                coreSize.setText("");
                coreAuthor.setText("");
            }
            catch (CompilerException e) {
		String error = e.getMessage();
		error = error.substring( 36, error.length() );
                coreDialogStatus.append( error );
                coreName.setText("");
                coreSize.setText("");
                coreAuthor.setText("");
            }
            CoreDialog.this.compile.setEnabled(true);
            CoreDialog.this.cancel.setEnabled(true);
            CoreDialog.this.loadCoreButton.setEnabled(true);            
            CoreDialog.this.coreFilename.setEnabled(true);
        }

    } // OKListener


    class cancelListener implements ActionListener {

        public void actionPerformed(ActionEvent ae) {
            // Dialog beenden
            CoreDialog.this.dispose();
        }

    } // CancelListener

    /**
     * Liefert den compilierten Core.
     * 
     * @return Der compilierte Core oder null falls noch keiner vorhanden
     */
    public CoreInfo getCoreInfo() {

        return coreInfo;

    } // getCoreInfo
    
    /**
     * Bringt den Dialog zur Anzeige.
     * 
     * @param adapter  WindowListener fr den Dialog
     */
    public void show(WindowAdapter adapter) {

        addWindowListener( adapter );
    	setVisible(true);

    } // show()

} // CoreDialog