/*
 * @(#)CoreWarClient.java    07.07.1999
 */
 
package corewar.client;

import corewar.CoreInfo;
import corewar.ParamInfo;
import corewar.GameSetupInfo;
import corewar.MessageBox;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.File;
import java.io.IOException;

/**
 * Die Klasse stellt das Hauptmen des Client-Programms dar.
 *
 * @author Hendrick Kay
 */
public class CoreWarClient extends Frame {

    /** @serial Ein Core-Auswahl-Dialog */
    private CoreDialog coreDialog;

    /** @serial Ein Verbindungs-Dialog*/
    private ConnectionDialog connectionDialog;

    /** @serial History-Anzeige */
    private HistoryFrame historyFrame;

    /** @serial Simulationsanzeige */
    private SimulationFrame simFrame;

    /** @serial Das eigene CoreInfo-Objekt */
    private CoreInfo coreInfo = null;

    private Button connectButton;

    /**
     * Initialisiert das Hauptmen.
     * 
     */
    public CoreWarClient() {

        super("CoreWarClient");

        try {
            ClientProperties.load();
        }
        catch (IOException e) {
            System.out.println("Einstellungen konnten nicht geladen werden");
        }

       // WindowListener hinzufgen
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent we) {
                try {
                    ClientProperties.save();
                }
                catch (IOException e) {
                    System.out.println("Einstellungen konnten nicht gespeichert werden");
                }
                System.exit(0);
            }
        });

        // Panel fr Buttons
        Panel myPanel = new Panel( new GridLayout(0,1) );
        
        // Buttons erzeugen
        Button coreDialogButton  = new Button("Core auswhlen");
        connectButton     = new Button("Verbinden");
        Button viewHistoryButton = new Button("History anschauen");
        Button quitButton        = new Button("Beenden");

        connectButton.setEnabled(false);

        // ActionListener fr Buttons erzeugen
        coreDialogButton.addActionListener( new CoreDialogListener() );
        connectButton.addActionListener( new ConnectionDialogListener() );
        viewHistoryButton.addActionListener( new ViewHistoryListener() );
        quitButton.addActionListener( new QuitListener() );

        // Buttons dem Panel hinzufgen
        myPanel.add(coreDialogButton);
        myPanel.add(connectButton);
        myPanel.add(viewHistoryButton);
        myPanel.add(quitButton);

        // Panel im Frame legen
        add(myPanel,"Center");
        
        // Hauptmen soll nicht in der Gre verndert werden knnen.
        setResizable(false);
        
        pack();
        centerScreen();
    }

    private void centerScreen() {
        Dimension dim = getToolkit().getScreenSize();
        Rectangle abounds = getBounds();
        setLocation((dim.width - abounds.width) / 2,
                    (dim.height - abounds.height) / 2);
    }

    class CoreDialogListener implements ActionListener {

        public void actionPerformed(ActionEvent ae) {

            // Hauptmen ausblenden
            CoreWarClient.this.setVisible(false);

            // neuen Dialog erstellen
	        coreDialog = new CoreDialog();

            // Core-Auswahl-Dialog einblenden
            coreDialog.show( new WindowAdapter() {

                    public void windowClosed(WindowEvent we) {
                        coreDialog.setVisible(false);
                        CoreWarClient.this.setVisible(true);
                        if ( coreDialog != null ) {
                            CoreWarClient.this.coreInfo = coreDialog.getCoreInfo();
                        }

                        if (CoreWarClient.this.coreInfo != null ) {
                            CoreWarClient.this.connectButton.setEnabled(true);
                        }
                        else {
                            CoreWarClient.this.connectButton.setEnabled(false);
                        }
                    }
                    public void windowClosing(WindowEvent we) {
                        coreDialog.setVisible(false);
                        CoreWarClient.this.setVisible(true);
                        if ( coreDialog != null ) {
                            CoreWarClient.this.coreInfo = coreDialog.getCoreInfo();
                        }

                        if (CoreWarClient.this.coreInfo != null ) {
                            CoreWarClient.this.connectButton.setEnabled(true);
                        }
                        else {
                            CoreWarClient.this.connectButton.setEnabled(false);
                        }
                    }
                    }
            ); // Ende CoreDialog einblenden

        }
    } // CoreDialogListener

    class ConnectionDialogListener implements ActionListener {

        public void actionPerformed(ActionEvent ae) {

            // CoreInfo aus dem CoreDialog holen

            CoreWarClient.this.coreInfo = coreDialog.getCoreInfo();            

            if ( coreInfo != null ) {

                // Hauptmen ausblenden
                CoreWarClient.this.setVisible(false);

                // neuen Dialog erstellen
                connectionDialog = new ConnectionDialog( coreInfo );
        
                // Verbindungsdialog einblenden;
                connectionDialog.show( new WindowAdapter() {
                    public void windowClosed(WindowEvent we) {
                        connectionDialog.setVisible(false);
                        GameSetupInfo gsi = connectionDialog.getGameSetup();
                        ClientServerSocket sock = connectionDialog.getServerSocket();
                        if (gsi != null) {
                            simFrame = new SimulationFrame(sock, gsi);
                            simFrame.show(new WindowAdapter() {
                                public void windowClosing(WindowEvent we) {
                                    CoreWarClient.this.simFrame.setVisible(false);
                                    CoreWarClient.this.setVisible(true);
                                }
                                }
                            );
                            simFrame.startSimulation();
                        }
                        else {
                            CoreWarClient.this.setVisible(true);
                        }
                    }
                    public void windowClosing(WindowEvent we) {
                        connectionDialog.setVisible(false);
                        GameSetupInfo gsi = connectionDialog.getGameSetup();
                        ClientServerSocket sock = connectionDialog.getServerSocket();
                        if (gsi != null) {
                            simFrame = new SimulationFrame(sock, gsi);
                            simFrame.show(new WindowAdapter() {
                                public void windowClosing(WindowEvent we) {
                                    CoreWarClient.this.simFrame.setVisible(false);
                                    CoreWarClient.this.setVisible(true);
                                }
                                }
                            );
                            simFrame.startSimulation();
                        }
                        else {
                            CoreWarClient.this.setVisible(true);
                        }
                    }

                    }
                );
            }
            else {
                MessageBox mbox = new MessageBox("Sie mssen erst einen Core kompilieren!",
                                                 "Fehler",
                                                 CoreWarClient.this,
                                                 MessageBox.OK);
                mbox.execute();
            }
 
         }

    } // ConnectListener

    class ViewHistoryListener implements ActionListener {

        public void actionPerformed(ActionEvent ae) {

            FileDialog fileDialog = new FileDialog(CoreWarClient.this,
                                                   "History auswhlen",
                                                   FileDialog.LOAD );
            // Datei-Dialog anzeigen
            fileDialog.show();

            // gewhlten Dateinamen ermitteln
            String filename = fileDialog.getFile();
            String path     = fileDialog.getDirectory();

            // wurde "CANCEL" geklickt?
            if ( filename != null ) {

                // Datei-Objekt erzeugen
                File histFile = new File(path, filename);
                // prfen, ob Datei existiert
                if ( histFile.exists() ) {

                    try {
                        historyFrame = new HistoryFrame(histFile);


                        // Hauptmen ausblenden
                        CoreWarClient.this.setVisible(false);

                        historyFrame.loadHistory();
                        historyFrame.show( new WindowAdapter() {
                                public void windowClosing(WindowEvent we) {
                                    CoreWarClient.this.historyFrame.setVisible(false);
                                    CoreWarClient.this.setVisible(true);
                                }

                                public void windowClosed(WindowEvent we) {
                                    CoreWarClient.this.historyFrame.setVisible(false);
                                    CoreWarClient.this.setVisible(true);
                                }
                           }
                        );
                    }
                    catch(IOException e) {}
                    catch(Exception e) {}
                }
                else {
                    MessageBox mbox = new MessageBox("Datei existiert nicht",
                                                     "Fehler",
                                                     CoreWarClient.this,
                                                     MessageBox.OK);
                    mbox.execute();
                }
            }

        }

    } // ViewHistoryListener

    class QuitListener implements ActionListener {

        // Programm beenden
        public void actionPerformed(ActionEvent ae) {
                try {
                    ClientProperties.save();
                }
                catch (IOException e) {
                    System.out.println("Einstellungen konnten nicht gespeichert werden");
                }
			System.exit(0);
        }

    } // QuitListener


    public static void main(String[] args) {

        CoreWarClient MyClient = new CoreWarClient();
        MyClient.setVisible(true);
    }

} // CoreWarClient
