/*
 * @(#)SimulationFrame.java    15.6.1999
 */


package corewar.client;

import corewar.MessageBox;
import corewar.ParamInfo;
import corewar.CoreInfo;
import corewar.GameInfo;
import corewar.GameSetupInfo;
import java.awt.Checkbox;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.io.IOException;
import java.net.InetAddress;
import java.util.Vector;

/**
 * Die Klasse SimulationFrame stellt das Anzeigefenster fr die CoreWar-
 * Simulation dar.
 *
 * @author  Ivo Kasiuk
 */
public class SimulationFrame extends ViewerFrame {

    /* Die Netzwerkverbindung zum Server. */
    private ClientServerSocket server;

    /* Die "Runde speichern"-Checkbox. */
    private Checkbox saveCheck;

    /* Signalisiert, da das Spiel beendet ist. */
    private boolean finished = false;
    
    /**
     * Initialisiert den SimulationFrame.
     * @param server     Der Server, der die Simulation durchfhrt.
     * @param gameSetup  Die Spieleinstallungen fr dieses Spiel.
     */
    public SimulationFrame(ClientServerSocket server, GameSetupInfo gameSetup) {
        super(gameSetup);
        this.server = server;
        status.setServer(server.getSocket().getInetAddress().getHostName());

        saveCheck = new Checkbox("Runde speichern");
        toolPanel.add(saveCheck);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent event) {
                try {
                    SimulationFrame.this.server.abortConnection();
                } catch (IOException ioe) {}
                abortGame();
                dispose();
            }
        });
        pack();
        centerScreen();
    }

    private void centerScreen() {
        Dimension dim = getToolkit().getScreenSize();
        Rectangle abounds = getBounds();
        setLocation((dim.width - abounds.width) / 2,
                    (dim.height - abounds.height) / 2);
    }

    /**
     * Signalisiert dem Server, da die Simulation beginnen kann.
     */
    public void startSimulation() {
        try {
            server.start(this);
        } catch (IOException ioe) {
            finished = true;
            (new MessageBox("Die Simulation konnte nicht gestartet werden:\n"
                            + ioe.getMessage(), "Fehler", this, MessageBox.OK))
                    .execute();
        }
    }

    /**
     * Verarbeitet einen Spielzyklus und spielt ihn ab.
     * @param pc            Die neuen PCs der Cores.
     * @param writeAddress  Die Schreibadressen der Cores.
     * @param writeLength   Die Lngen der Schreibzugriffe.
     */
    public void handleCycle(short[] pc, short[] writeAddress,
                            byte[] writeLength) {
        history.addCycle(pc, writeAddress, writeLength);
        cycleForward();
    }

    /**
     * Startet das Speichern der Spielrunde, falls die "Runde speichern"-
     * Checkbox aktiviert ist und fhrt die Rundenauswertung durch.
     * @return  true, wenn das die letzte Runde war.
     */
    public boolean endRound() {
        GameInfo gameInfo = status.getGameInfo();
        if (gameInfo.getRoundNumber() == gameInfo.getTotalRounds()) {
            finished = true;
        }
        if (saveCheck.getState()) {
            saveHistory();
        } else {
            try {
                server.readyForData();
            } catch (IOException ioe) {}
        }
        return super.endRound();        
    }

    /**
     * Signalisiert dem Server ggf., da die nchste Spielrunde beginnen kann.
     */
    protected void savingFinished() {
        if (!(finished || playing) && (saveThreads == 0)) {
            try {
                server.readyForData();
            } catch (IOException ioe) {}
        }
    }

    /**
     * Bricht ein laufendes Spiel ab. Falls gerade eine Spielrunde luft, kann
     * sie noch gespeichert werden.
     */
    void abortGame() {
        finished = true;
        if (playing && saveCheck.getState()) {
            saveHistory();
            playing = false;
        }
    }

    public static void main(String[] args) {
        ParamInfo param = new ParamInfo();
        param.setRAMSize(8192);
        Vector cores = new Vector();
        CoreInfo core = new CoreInfo();
        core.coreName = "Super Core";
        core.author = "Coremaster";
        core.size = 364;
        cores.addElement(core);
        core = new CoreInfo();
        core.coreName = "Raupe";
        core.author = "Anonymous";
        core.size = 6;
        cores.addElement(core);
        core = new CoreInfo();
        core.coreName = "Dabeiseinisalles";
        core.author = "Loser";
        core.size = 142;
        cores.addElement(core);
        for (int i=0; i<17; i++) {
            core = new CoreInfo();
            core.coreName = "TestCore";
            core.author = "TestAutor";
            core.size = 123;
            cores.addElement(core);
        }
        GameSetupInfo setup = new GameSetupInfo(param, cores, (byte) 0);
        SimulationFrame viewer = new SimulationFrame(null, setup);
        viewer.show(new WindowAdapter() {
            public void windowClosed(WindowEvent event) {
                System.exit(0);
            }
        });
    }
}
