/*
 * @(#)StepHistory.java    06.07.1999
 */

package corewar.client;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;

import java.lang.reflect.Array;

/**
 * Die Klasse ermglicht das Speichern einer ganzen Spielrunde. Zustzlich
 * kann sie die Spielrunde im Einzelschrittmodus vorwrts und rckwrts
 * abspielen.
 *
 * @author Hendrick Kay
 */
public class StepHistory {

    /** Darstellungsobjekt des Spielspeichers */
    private RAMDisplay ram;

    /** Gre eines dynamischen Speicherblocks */
    private int blockSize;

    /** Array mit den neuen PCs fr jeden Step */
    private short[][] pc;
    
    /** Array mit der Schreibadresse fr jeden Step */
    private short[][] writeAddress;

    /** Array mit der Schreiblnge fr jeden Step */
    private byte[][] writeLength;

    /** Array mit der Corenummern fr die Vorgngerfarbe fr jeden Step */
    private byte[][] preColor;

    /** Array mit AnfangsPCs aller Cores */
    private int[] startPC;

    /* folgende Instanzvariablen dienen zur Speicherung eines einzelnen Zyklus
     * So kann das Spiel trotzdem weiterhin angezeigt werden auch, wenn
     * ( enoughMem == false ) gilt.
     */

    /** Array fr Schreibadressen */
    private short[] addressBuffer;

    /** Array fr Schreiblngen */
    private byte[] lengthBuffer;

    /** Array fr PC-Adressen */
    private short[] pcBuffer;

    /** Array fr Vorgngerfarben */
    private byte[] colorBuffer;

    /** Array mit den noch lebenden Cores */
    private boolean[] aliveCores;

    /** Core der diesen Step ausfhrt */
    private byte core;

    /** Zyklennummern in den denen die Cores gestorben sind */
    private int[] coreDeath;

    /** aktueller Index auf writeAddress und writeLength */
    private int stepPos;

    /** aktueller Index auf preColor */
    private int colorPos;

    /** aktuelle Abspielposition in writeAddress und writeLength */
    private int stepPlayPos;

    /** aktuelle Abspielposition in preColor */
    private int colorPlayPos;

    /** Begrenzung von stepPos */
    private int maxStepPos;

    /** Begrenzung von colorPos */
    private int maxColorPos;

    /** gibt an, ob genug Speicher worhanden ist */
    private boolean enoughMem;

    /** Anzahl der Cores */
    private byte cores;

    /**
     * Initialisiert die StepHistory mit Spieldaten.
     *
     * @param objectInStream  Eingabestrom fr Spieldatn.
     * @param ram             Spielanzeige.
     * @param maxCycles       Anzahl der maximalen Zyklen.
     * @exception IOException
     */
    public StepHistory(ObjectInputStream objectInStream,
                        RAMDisplay ram, int maxCycles) {

            try {
                startPC = (int[]) objectInStream.readObject();

                // RAMDisplay speichern
                this.ram = ram;

                // StartPCs der Cores setzen
                this.ram.setStartPC(startPC);

                // RAMDisplay in den Anfangszustand versetzen
                this.ram.reset();

                // Anzahl der Cores ermitteln
                cores = (byte) startPC.length;

                // aktuellen Core setzen
                core = 0;

                // Aufzeichnungspositionen in den Arrays
                System.out.print("lese stepPos: ");
                stepPos = ( (Integer) objectInStream.readObject()).intValue();
                System.out.println(stepPos);

                System.out.print("lese colorPos: ");
                colorPos = ( (Integer) objectInStream.readObject()).intValue();
                System.out.println(colorPos);

                // maximale Aufzeichnungsposition fr pc, writeAddress, writeLength
                System.out.print("lese maxStepPos: ");
                maxStepPos = ( (Integer) objectInStream.readObject()).intValue();
                System.out.println(maxStepPos);

                // maximale Aufzeichnungsposition fr preColor
                System.out.print("lese maxColorPos: ");
                maxColorPos = ( (Integer) objectInStream.readObject()).intValue();
                System.out.println(maxColorPos);

//                coreDeath = (int[]) objectInStream.readObject();

                int numberOfArrays;
                numberOfArrays = ( (Integer) objectInStream.readObject()).intValue();

                // Anzahl der Steps berechnen
                int maxSteps = maxCycles * cores;

                // passende Blockgre errechnen;
                if ( maxSteps > 50000 ) {

                    // Blockgre so berechnen, da ein Block
                    // immer fr ganze Zyklen Daten fassen kann
                    blockSize = 50000-50000%cores;

                    // Arrays initialsieren
                    pc           = new short[ (maxSteps/blockSize) + 1 ][];
                    writeAddress = new short[ (maxSteps/blockSize) + 1 ][];
                    writeLength  = new byte[ (maxSteps/blockSize) + 1 ][];
                    // vorherige Farben knnen bis zu 6 mal mehr Platz beanspruchen
                    preColor = new byte[ ( 6 * (maxSteps/blockSize)) + 1 ][];
                }
                else {

                    // alle mglichen Schritte in einen Block
                    blockSize = maxSteps;

                    // Arrays initialsieren
                    pc           = new short[1][];
                    writeAddress = new short[1][];
                    writeLength  = new byte[1][];

                    // vorherige Farben knnen bis zu 6 mal mehr Platz beanspruchen
                    preColor = new byte[1][];
                }


                // Arrays lesen (pc,writeAddress, writeLength)
                System.out.println("lese normal-Arrays");
                for (int i=0; i < numberOfArrays; i++) {

                    pc[i]           = (short[]) objectInStream.readObject();
                    writeAddress[i] = (short[]) objectInStream.readObject();
                    writeLength[i]  = (byte[]) objectInStream.readObject();

                }

                numberOfArrays = ( (Integer) objectInStream.readObject()).intValue();

                // Arrays lesen (preColor)
                System.out.println("lese color-Arrays");
                for (int i=0; i < numberOfArrays; i++) {
                    preColor[i] = (byte[]) objectInStream.readObject();
                }

            }
            catch (IOException e) {
                System.out.println("FEHLER: Ein-/Ausgabefehler beim Laden" + e);
            }
            catch (Exception e) {
                System.out.println("FEHLER: Ladefehler " + e);
            }

            // Abspielposition in den Arrays
            stepPlayPos = 0;
            colorPlayPos = 0;

            // alle cores als lebend markieren
            aliveCores = new boolean[cores];
            for (int i=0; i < cores; i++) { aliveCores[i] = true; }

            // Todeszeitpunkte initialisieren, -1 ist unbekannter Todeszeitpunkt
            coreDeath = new int[cores];
            for (int i=0; i < cores; i++) { coreDeath[i] = -1; }

            // noch haben wir genug Speicher;
            enoughMem = true;
    }

    /**
     * Initialisiert eine spieldatenlose StepHistory.
     *
     * @param ram             Spielanzeige.
     * @param corePC          Array mit den Startadressen der Cores.
     * @param maxCycles       Anzahl der maximalen Zyklen.
     */
    public StepHistory(RAMDisplay ram, int[] corePC, int maxCycles) {

            // RAMDisplay speichern
            this.ram = ram;

            // StartPCs der Cores setzen
            this.ram.setStartPC(corePC);

            // RAMDisplay in den Anfangszustand versetzen
            this.ram.reset();

            // Anzahl der Cores ermitteln
            cores = (byte) corePC.length;

            // aktuellen Core setzen
            core = 0;

            // StartPCs einrichten
            startPC = corePC;

            // Anzahl der Steps berechnen
            int maxSteps = maxCycles * cores;

            // passende Blockgre errechnen;
            if ( maxSteps > 50000 ) {

                // Blockgre so berechnen, da ein Block
                // immer fr ganze Zyklen Daten fassen kann
                blockSize = 50000-50000%cores;

                // Arrays initialsieren
                pc           = new short[ (maxSteps/blockSize) + 1 ][];
                writeAddress = new short[ (maxSteps/blockSize) + 1 ][];
                writeLength  = new byte[ (maxSteps/blockSize) + 1 ][];
                // vorherige Farben knnen bis zu 6 mal mehr Platz beanspruchen
                preColor = new byte[ ( 6 * (maxSteps/blockSize)) + 1 ][];
            }
            else {

                // alle mglichen Schritte in einen Block
                blockSize = maxSteps;

                // Arrays initialsieren
                pc           = new short[1][];
                writeAddress = new short[1][];
                writeLength  = new byte[1][];

                // vorherige Farben knnen bis zu 6 mal mehr Platz beanspruchen
                preColor = new byte[1][];
            }

            // Aufzeichnungspositionen in den Arrays
            stepPos = -1;                // -1 heit es gibt noch keine Steps
            colorPos = -1;

            // Abspielposition in den Arrays
            stepPlayPos = 0;
            colorPlayPos = 0;

            // maximale Aufzeichnungsposition fr pc, writeAddress, writeLength
            maxStepPos = -1;
            
            // maximale Aufzeichnungsposition fr preColor
            maxColorPos = -1;

            // alle cores als lebend markieren
            aliveCores = new boolean[cores];
            for (int i=0; i < cores; i++) { aliveCores[i] = true; }

            // Todeszeitpunkte initialisieren, -1 ist unbekannter Todeszeitpunkt
            coreDeath = new int[cores];
            for (int i=0; i < cores; i++) { coreDeath[i] = -1; }

            // noch haben wir genug Speicher;
            enoughMem = true;
    }


    /**
     * Speichert die Spieldaten.
     *
     * @param objectOutStream  Ausgabestrom in den gespeichert werden soll.
     */
    public void save(ObjectOutputStream objectOutStream) {

        int numberOfArrays = 0;
        try {        
            System.out.println("SPEICHERN:\n");
            // StartPCs speichern
            objectOutStream.writeObject(startPC);

            // Positionsvariablen speichern
            System.out.print("schreibe stepPos: ");
            objectOutStream.writeObject(new Integer(stepPos));
            System.out.println(stepPos);

            System.out.print("schreibe colorPos: ");
            objectOutStream.writeObject(new Integer(colorPos));
            System.out.println(colorPos);

            System.out.print("schreibe maxStepPos: ");
            objectOutStream.writeObject(new Integer(maxStepPos));
            System.out.println(maxStepPos);

            System.out.print("schreibe maxColorPos: ");
            objectOutStream.writeObject(new Integer(maxColorPos));
            System.out.println(maxColorPos);

//            System.out.println("schreibe coreDeath");
//            objectOutStream.writeObject(coreDeath);

            // Feststellen wieviele Arrays zu speichern sind
            numberOfArrays = stepPos/blockSize;
            if ( stepPos%blockSize != 0) {
                numberOfArrays++;
            }

            // Anzahl der Arrays speichern
            objectOutStream.writeObject(new Integer(numberOfArrays));
            System.out.println("number of normal Arrays: " + numberOfArrays);
            // Arrays speichern (pc,writeAddres, writeLength)
            for (int i=0; i < numberOfArrays; i++) {
                objectOutStream.writeObject(pc[i]);
                objectOutStream.writeObject(writeAddress[i]);
                objectOutStream.writeObject(writeLength[i]);
            }

            // Feststellen wieviele Arrays zu speichern sind
            numberOfArrays = 0;
            numberOfArrays = colorPos/(6*blockSize);
            if ( colorPos%(6*blockSize) != 0) {
                numberOfArrays++;
            }

            // Anzahl der Arrays speichern
            objectOutStream.writeObject(new Integer(numberOfArrays));
            System.out.println("number of color Arrays: " + numberOfArrays);
            
            // Arrays speichern (preColor)
            for (int i=0; i < numberOfArrays; i++) {
                objectOutStream.writeObject(preColor[i]);
            }
        }
        catch (IOException e) {
            System.out.println("FEHLER: Ein-/Ausgabefehler beim Speichern");
        }

    } // save()

    /**
     * Fgt die Spieldaten eines Zyklus hinzu.
     *
     * @param pc         Die neuen PC-Adressen.
     * @param writeAddr  Schreibadressen der Cores.
     * @param length     Schreiblngen der Cores.
     */

    public void addCycle(short[] pc, short[] writeAddr, byte[] length) {

        if (enoughMem) {

            long freeMem = 1000000;

            // evtl. Array vergrern
            if ( stepPos + pc.length > maxStepPos ) {

                // noch genug Speicher da?
                if ( freeMem > 6*blockSize ) {
                    int block = ( stepPos + pc.length ) / blockSize;
                    this.pc[block] = new short[blockSize];
                    writeAddress[block] = new short[blockSize];
                    writeLength[block] = new byte[blockSize];

                    // neue Grenze setzen
                    maxStepPos+=blockSize;
                }
                else {
                    System.out.println("FEHLER in addcycle: nicht genug Speicher");
                    enoughMem = false;

                    // Anfangspositionen fr Buffer setzen
                    stepPlayPos = 0;
                    colorPlayPos = 0;
                }
            }

            // feststellen, um wieviel colorPos erhht werden mu
            byte preColorSpace = 0;
            for (int i=0; i<Array.getLength(length); i++) {
                preColorSpace+=length[i];
            }

            // evtl. Array vergrern
            if ( (colorPos + preColorSpace) > maxColorPos ) {

                // noch genug Speicher da?
                if (freeMem > 6*blockSize) {
                    int block = (colorPos + preColorSpace) / (6 * blockSize);
                    preColor[block] = new byte[6 * blockSize];

                    // neue Grenze setzen
                    maxColorPos += 6*blockSize;
                }
                else {
                    System.out.println("FEHLER in addcycle: nicht genug Speicher");
                    enoughMem = false;
                    // Anfangspositionen fr Buffer setzen
                    stepPlayPos = 0;
                    colorPlayPos = 0;
                    // TODO: alles fr Speicher-knapp-Modus einrichten
                }
            }
        }

        byte leng = 0;
        
        // sind wir im Speicher-knapp-Modus?
        if (enoughMem) {

            // Arrays mit Werten fllen
            for (int i=0; i<pc.length; i++) {

                // ArrayPosition erhhen
                stepPos++;

                int blockIndex = stepPos / blockSize;
                int arrayIndex = stepPos % blockSize;

                // PCs kopieren
                this.pc[blockIndex][arrayIndex] = pc[i];

                // Schreibadressen kopieren
                writeAddress[blockIndex][arrayIndex] = writeAddr[i];

                // Schreiblngen
                leng = length[i];
                if ( ( leng < -1 ) || ( leng > 6 ) ) {
                    System.out.println("illegale Schreiblnge: " + leng);
                    leng = 0;
                }
                else {
                    writeLength[blockIndex][arrayIndex] = leng;
                }
            }

            // die Corenummern der Vorgngerfarben ermitteln
            for (int i=0; i<writeAddr.length; i++) {

                for (int n=0; n<length[i]; n++) {

                    // ArrayPosition erhhen
                    colorPos++;
                    
                    // Vorgngerfarbe kopieren
                    int block = colorPos / ( 6 * blockSize );
                    int index = (colorPos % ( 6 * blockSize ) );
                    byte preCore = ram.getByte( sshort2uint(writeAddr[i])+n );
                    preColor[block][index] = preCore;
                }
            }

        }
        else {
            pcBuffer      = pc;
            addressBuffer = writeAddr;
            lengthBuffer  = length;
        }

    } // addCycle()

    /**
     * Setzt die Abspielposition wieder an den Anfang und aktualisiert
     * die Spielanzeige.
     *
     */
    public synchronized void reset() {
        System.out.println("reset() called");

        // Abspielposition an den Anfang setzen
        stepPlayPos=0;
        colorPlayPos=0;

        // alle cores als lebend markieren
        aliveCores = new boolean[cores];
        for (int i=0; i < cores; i++) { aliveCores[i] = true; }

        // Todeszeitpunkte initialisieren, -1 ist unbekannter Todeszeitpunkt
        coreDeath = new int[cores];
        for (int i=0; i < cores; i++) { coreDeath[i] = -1; }

        // StartPCs der Cores setzen
        this.ram.setStartPC(startPC);
                
        // RAMDisplay in den Anfangszustand versetzen
        this.ram.reset();

        enoughMem = true;

    } // reset()

    /**
     * Fhrt einen Vorwrtsschritt in der StepHistory aus.
     *
     * @return boolean bestimmt, ob der Core des Steps gestorben ist
     */
    public synchronized boolean stepForward() {

//        System.out.println("stepForward for Core:" + core);
        // sind wir im genug-Speicher-Modus?
        if (enoughMem) {

            if ( stepPlayPos <= stepPos ) {

                // Parameter fr RAMDisplay.setBytes() berechnen
                short address = getHistShort(writeAddress, stepPlayPos);
                byte writeLen = getHistByte(writeLength, stepPlayPos);

                // Vorgngerfarben kopieren
                byte preCore;
                for (int i=0; i<writeLen; i++) {
                    preCore = ram.getByte( sshort2uint(address)+i );
                    setHistByte(preColor, colorPlayPos + i, preCore );
                }

                switch (writeLen) {
                    case  6:
                    case  5:
                    case  4:
                    case  3:
                    case  2:
                    case  1:
                            // Schreiboperation durchfhren
                            ram.setBytes(core, sshort2uint(address) , writeLen);

                            // Arrayindex anpassen
                            colorPlayPos+=writeLen;
                            break;
                    case  0:
                            break;
                    case -1:
                            try {
                                System.out.println("Core " + core + "died");
                                coreDeath[core] = stepPlayPos;
                                aliveCores[core] = false;
                            }
                            catch (ArrayIndexOutOfBoundsException e) {
                                System.out.println("Fehler: stepForward(): " + core);
                            }
                            break;
                    default:
                            System.out.println("FEHLER: illegale Schreiblnge " + writeLen);
                }
            }
            else {
                System.out.println("FEHLER: keine Spieldaten vorhanden");
                return false;
            }

            ram.setPC(core, sshort2uint(getHistShort(pc, stepPlayPos)) );

            stepPlayPos++;
        }
        else {
            // Parameter fr RAMDisplay.setBytes() berechnen
            short address = addressBuffer[stepPlayPos];
            byte writeLen = lengthBuffer[stepPlayPos];

            // RAMDisplay updaten
            if (writeLen > 0) {
                ram.setBytes(core, sshort2uint(address) , writeLen);
            }
            else {
                // ist der Core gestorben?
                if (writeLen == -1) {
                    coreDeath[core] = stepPlayPos;
                    // TODO: PC aus Bildschirm entfernen
                }
                else {
                    if ( writeLen != 0 ) {
                        System.out.println("FEHLER: illegale Schreiblnge " +
                            writeLen);
                    }
                }

            }
            ram.setPC(core, sshort2uint(pcBuffer[stepPlayPos]) );
            stepPlayPos++;
            if (stepPlayPos==cores) { stepPlayPos = 0; }

        }

        // feststellen, ob der Core dieses Steps gestorben ist
        if ( (stepPlayPos-1) == coreDeath[core] ) {
            // nchsten Core ermitteln
            nextCore();
            return true;
        }
        else {
            // nchsten Core ermitteln
            nextCore();
            return false;
        }

    } // stepForward

    /**
     * Fhrt einen Rckwrtsschritt in der StepHistory aus.
     * @return int wiederzubelebender Core oder -1
     *
     */
    public synchronized int stepBackward() { 

        int reviveCore = -1;

        if ( stepPlayPos > 0 ) {

            if (enoughMem) {

                // einen Step zurck
                stepPlayPos--;
                // letzen Core setzen
                prevCore();

                // letzte Schreibadresse holen
                short address = getHistShort(writeAddress, stepPlayPos);
                // letzte Schreiblnge holen
                byte writeLen = getHistByte(writeLength, stepPlayPos);

                switch( writeLen ) {
                    case  6:
                    case  5:
                    case  4:
                    case  3:
                    case  2:
                    case  1:
                        // Schreiboperation rckgngig machen (max 6 Farben)
                        colorPlayPos-=writeLen;
                        for (int i=0; i<writeLen; i++) {
                            ram.setByte(getColorHistByte(preColor, colorPlayPos ), sshort2uint(address)+i );
                            colorPlayPos++;
                        }
                        colorPlayPos-=writeLen;
                        break;
                    case  0:
                        break;
                    case -1:        
                        // feststellen welcher core wiederbelebt werden mu
                        for (core = 0; core < coreDeath.length;core++) {
                            if (coreDeath[core] == stepPlayPos) {
                                break;
                            }
                        }
                        aliveCores[core] = true;
                        reviveCore = core;

                        break;
                    default:
                        System.out.println("Fehler: illegale Schreiblngen in History");
                }

//                System.out.println("stepBackward for Core:" + core);

                // letzten PC neuzeichnen und alten entfernen
                int newPC;
                if ( (stepPlayPos-1) < cores) {
                    newPC = startPC[core];
                }
                else {
                    newPC = sshort2uint( getHistShort(pc, stepPlayPos-1) );
                }
                ram.setPC(core, newPC );

            }
            else {
                System.out.println("Fehler: nicht genug Speicher fr History-Funktion");
            }
        }
        else {
            System.out.println("Fehler: stepPlayPos = " + stepPlayPos);
        }
        return reviveCore;

    } // stepBackward

    /**
     * Wandelt einen signed short in ein unsigned int um.
     *
     * @param number  das umzuwandelnde short
     * @return int    das umgewandelte int
     */
    private int sshort2uint(short number) {
       return ( (int) number ) & 65535;
    } // sshort2uint()

    /**
     * Liefert das Byte an Spielposition Pos.
     *
     * @param array  das Array aus dem gelesen werden soll.
     * @param Pos    Spielposition.
     */
    private short getHistShort(short[][] array, int Pos) {
        short a;
        try {
            a = array[ Pos / blockSize ][Pos % blockSize];
        }
        catch (ArrayIndexOutOfBoundsException e) {
            System.out.println("Fehler: getHistShort()" + Pos / blockSize + " " + Pos % blockSize);
            a = 0;
        }
        return a;
    } // getHistShort()

    /**
     * Liefert das Byte an Spielposition Pos im array.
     *
     * @param array  das Array aus dem gelesen werden soll.
     * @param Pos    Spielposition.
     */
    private byte getHistByte(byte[][] array, int Pos) {
        byte a;
        try {
            a = array[ Pos / blockSize ][Pos % blockSize];
        }
        catch (ArrayIndexOutOfBoundsException e) {
            System.out.println("Fehler: getHistByte() " + Pos / blockSize + " " + Pos % blockSize);
            a = 0;
        }
        return a;
    } // getHistByte()

    /**
     * Liefert das Byte an Spielposition Pos im array.
     *
     * @param array  das Array aus dem gelesen werden soll.
     * @param Pos    Spielposition.
     */
    private byte getColorHistByte(byte[][] array, int Pos) {
        byte a;
        try {
            a = array[ Pos / (6*blockSize) ][Pos % (6*blockSize) ];
        }
        catch (ArrayIndexOutOfBoundsException e) {
            System.out.println("Fehler: getHistByte() " + Pos / blockSize + " " + Pos % blockSize);
            a = 0;
        }
        return a;
    } // getColorHistByte()

    /**
     * setzt das Byte an Spielposition Pos im array.
     *
     * @param array       das Array aus dem gelesen werden soll.
     * @param Pos         Spielposition.
     * @param coreNumber  das zu schreibende Byte.
     */
    private void setHistByte(byte[][] array, int Pos, byte coreNumber) {
        try {
            array[ Pos / (6*blockSize) ][Pos % (6*blockSize) ] = coreNumber;
        }
        catch (ArrayIndexOutOfBoundsException e) {
            System.out.println("Fehler: setHistByte() " + Pos / blockSize + " " + Pos % blockSize);
        }
    } // getHistByte()


    /**
     * Liefert den Core fr den nchsten Step
     */
    private byte nextCore() {

        // von ArrayPos core bis Ende nach lebenden Cores suchen
        byte index = (byte) (core+1);
        for (byte i = 0; i < cores; i++) {
            try {
                if ( aliveCores[index%cores]==true ) {
                    core = (byte) (index%cores);
                    return core;
                }
            }
            catch (ArrayIndexOutOfBoundsException e) {
                System.out.println("Fehler: nextCore() " + index%cores);
            }            
            index++;
        }

        // diese Befehle werden nie erreicht
        System.out.println("FEHLER: alle Cores sind tot");
        return -1;
    } // nextCore()


    public byte previousCore() {

        if (stepPlayPos != 0) {

            byte lastCore=0;
            // feststellen welcher core wiederbelebt werden mu
            for (lastCore = 0; lastCore < coreDeath.length;lastCore++) {
                if (coreDeath[lastCore] == stepPlayPos) {            
                    break;
                }
            }

            if ( coreDeath[lastCore] != stepPlayPos ) {
                byte index = (byte) (core - 1 + cores);
                for (byte i = 0; i < cores; i++) {
                    try {
                        if ( aliveCores[index%cores]==true ) {
                            lastCore = (byte) (index%cores);
                            return lastCore;
                        }
                    }
                    catch (ArrayIndexOutOfBoundsException e) {
                        System.out.println("Fehler: prevCore() " + index%cores);
                    }            
                    index--;
                }
            }
            else {
                return lastCore;
            }
        }
        else {
            return -1;
        }
        return -1;
    }

    /**
     * Liefert den Core fr den vorherigen Step.
     */
    private byte prevCore() {

        byte index = (byte) (core - 1 + cores);
        for (byte i = 0; i < cores; i++) {
            try {
                if ( aliveCores[index%cores]==true ) {
                    core = (byte) (index%cores);
                    return core;
                }
            }
            catch (ArrayIndexOutOfBoundsException e) {
                System.out.println("Fehler: prevCore() " + index%cores);
            }            
            index--;
        }

        // diese Befehle werden nie erreicht
        System.out.println("FEHLER: alle Cores sind tot");
        return -1;
    } // prevCore()


} // StepHistory
