package corewar.client;

import corewar.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class TestRAMDisplay extends Frame {

    private Button clearButton = new Button(" -x ");
    private Button drawButton  = new Button("draw");
    private Button fill        = new Button(" +x ");
    private Button cancel      = new Button("Abbrechen");
    private TextField address  = new TextField("1000");
    private TextField length   = new TextField("1");

    private RAMDisplay ram;
    private StepHistory history;

    static Random ran = new Random();


    public TestRAMDisplay(int size, int players) {

        super("TestRAMDisplay");

        // Unter Windows die Hintergrundfarbe korrigieren
        if (System.getProperty("os.name").startsWith("Windows")) {
            setBackground(SystemColor.control);
        }

        Panel panel = new Panel( new FlowLayout() );
// test start        
        // Vector mit CoreStatus
        Vector test = new Vector();
        // Vector mit StartPCs
        int[] startPC = new int[players];

        CoreStatus myC;
        Random myRandom = new Random();
        int myNum;
        int myPC;
        for (int i=0; i<players; i++) {
            myC = new CoreStatus();
            myPC = rand(size);
            myC.color = new Color( rand(256), rand(256), rand(256) );
            myC.pc = myPC;
 
            startPC[i] = myPC ;
            test.addElement( myC );
        }

        ram = new RAMDisplay( size, test );

        history = new StepHistory(ram, startPC, 10000);

        short[] pc = new short[players];
        short[] wa = new short[players];
        byte[]  wl = new byte[players];

        // 50000 Zyklen
        for (int i=0; i<10000;i++) {
            // #players Spieler
            for (int n = 0; n<players; n++) {
                pc[n] = randS(size);
                wa[n] = randS(size-6);
                wl[n] = (byte) (rand(6)+1);
            }
            history.addCycle(pc, wa, wl);
        }
// test end 
        address.setColumns(6);
        length.setColumns(6);
//        panel.add(length);
//        panel.add(drawButton);
        panel.add(clearButton);
        panel.add(address);
        panel.add(fill);
        panel.add(cancel);

        drawButton.addActionListener(new DrawListener() );
        clearButton.addActionListener(new ClearListener() );
        fill.addActionListener(new FillListener() );
        cancel.addActionListener(new CancelListener() );

        add(ram,"Center");
        add(panel,"South");
        pack();
//        setLocation(250,200);
        centerScreen();
    }


    private void centerScreen() {
        Dimension dim = getToolkit().getScreenSize();
        Rectangle abounds = getBounds();
        setLocation((dim.width - abounds.width) / 2,
                    (dim.height - abounds.height) / 2);
    }


    public static int rand(int n) {
        int Num;
        Num = ( ran.nextInt() % n );
        if ( Num < 0 ) Num*=-1;
        return Num;
    }


    public static short randS(int n) {
        int Num;
        Num = ( ran.nextInt() % n );
        if ( Num < 0 ) Num*=-1;
        if ( Num > 32767 ) {
            Num -= 65535;
        }
        return (short) Num;
    }


    class DrawListener implements ActionListener {
        public void actionPerformed(ActionEvent ae) {
            Integer a = new Integer(address.getText());
            Integer b = new Integer(length.getText());
            
            if ( b.intValue() == 1 ) {
                ram.setByte( (byte) 0, a.intValue() );
            }
            else {
                ram.setBytes( (byte) 0, a.intValue(), (short) b.intValue() );
            }
        }
    }

    class ClearListener implements ActionListener {
        public void actionPerformed(ActionEvent ae) {
            int a = new Integer(address.getText()).intValue();
            new backward().start(a);
        }
    }

    class FillListener implements ActionListener {
        public void actionPerformed(ActionEvent ae) {
            int a = new Integer(address.getText()).intValue();
            new forward().start(a);
        }
    }

    class forward extends Thread {

        public void start(int a) {
            run(a);
        }

        public void run(int a) {
            for (int i=0; i<a; i++) {
                if ( (i % 1000 == 0) ) System.out.println(i);
/*                try {
                    this.sleep(5);
                }

                catch (InterruptedException e) {}
*/
                history.stepForward();
            }
        }

    }

    class backward extends Thread {

        public void start(int a) {
            run(a);
        }

        public void run(int a) {
            for (int i=0; i<a; i++) {
                if ( (i % 1000 == 0) ) System.out.println(i);
/*                try {
                    this.sleep(5);
                }

                catch (InterruptedException e) {}
*/
                history.stepBackward();
            }
        }

    }


    class CancelListener implements ActionListener {
        public void actionPerformed(ActionEvent ae) {
            TestRAMDisplay.this.dispose();
        }
    }

    public void show(WindowAdapter adapter) {
         addWindowListener( adapter );
         setVisible(true);
    }
}