/*



 * @(#)Coder.java    20.06.1999



 */





/*



 * Diese Klasse produziert den Objektcode eines Cores aus den


 * ParsedLine-Objekten



 * @author Benjamin Stehr



 */









package corewar.compiler;

import java.util.Hashtable;

import java.util.Vector;

import corewar.CoreInfo;


public class Coder {



    // in labels werden gefundene Labels und deren Wert gespeichert

    
    Hashtable labels=new Hashtable();

    

    

    

    //  unresolved enthlt im Quelltext gefundene Referenzen auf Label.

    

    Vector unresolved=new Vector();

    

    



    //  Zhler fr die Position, an der weiterer Objektcode angefgt werden mu.

    

    short pc=0;

    

    



    // enthlt den Objektcode, der spter in die CoreInfo geschrieben wird.

    

    byte objectCode [] = new byte [1024];

    

    

    

    // reloc wird bentigt um im Interpreter feststellen zu knnen, an welcher Stelle im Objektcode Label auftreten.

    // Es wird immer die Position des Verweises und der Wert des Labels nacheinander gespeichert.

    

    int reloc [] = new int [512];





    // Konstanten fr die Befehle

    

    private static final int ADD=1;

    private static final int AND=2;

    private static final int BOZ=3;

    private static final int JMN=4;

    private static final int JMP=5;

    private static final int JMPatA=6;

    private static final int JMZ=7;

    private static final int MMR=8;

    private static final int MOH=9;

    private static final int MOV=10;

    private static final int MOVhashA=11;

    private static final int MOVatA=12;

    private static final int MOVatB=13;

    private static final int MUL=14;

    private static final int NEG=15;

    private static final int SIC=16;

    private static final int SLL=17;

    private static final int SRL=18;

    private static final int SUB=19;

    private static final int XOR=20;





    /** Produziert den Objektcode zu der Zeile line. Der Objektcode wird intern

    

    *   gespeichert, bis writeCode (CoreInfo coreInfo) aufgerufen wird.

    

    *   Ein Codierungsfehler fhrt zu einer CompilerException

    

    */



    public void codeLine (ParsedLine line) throws CompilerException{



        // falls die Zeile mit einem Label beginnt, wird der Name des Labels 

        // und die Position gespeichert, um spter das Reloc-Array erzeugen

        // zu knnen

	 

        if (line.label!=null) {

            if (labels.containsKey(line.label)) {

                throw new CompilerException ("Doppelt vergebenes Label");

            }

            else {

                labels.put (line.label, new Short(pc));

            }

        }



        

	if (pc>1017) {

            throw new CompilerException ("Maximale Core-Gre berschritten");

        }



        

// berprfung des Syntax der einzelnen Befehle



        if (line.operation==ParsedLine.OP_ADD) {

            if ((line.operandA==null) || (line.operandB==null)) {

                throw new CompilerException ("zwei Operanden fr ADD-Befehl erforderlich");

            }

            if ((line.operandA.addressMode!=Operand.MODE_DIRECT) ||

               (line.operandB.addressMode!=Operand.MODE_DIRECT)) {

                throw new CompilerException ("Nur direkte Adressierungen fr ADD-Befehl zugelassen");

            }

            if (line.offsetFound) throw new CompilerException

                ("<Offset> fr ADD-Befehl unzulssig");



            objectCode[pc]=ADD;

            pc++;

            objectCode [pc]=0;              //offset

            pc++;

            addObjectCode (line);



        }



        if (line.operation==ParsedLine.OP_AND) {

            if ((line.operandA==null) || (line.operandB==null)) {

                throw new CompilerException ("zwei Operanden fr AND-Befehl erforderlich");

            }

            if ((line.operandA.addressMode!=Operand.MODE_DIRECT) ||

               (line.operandB.addressMode!=Operand.MODE_DIRECT)) {

                throw new CompilerException ("Nur direkte Adressierungen fr AND-Befehl zugelassen");

            }

            if (line.offsetFound) throw new CompilerException

                ("<Offset> fr AND-Befehl unzulssig");



            objectCode[pc]=AND;

            pc++;

            objectCode [pc]=0;              //offset

            pc++;

            addObjectCode (line);



        }





        if (line.operation==ParsedLine.OP_BOZ) {



            if (line.operandB!=null) {

                throw new CompilerException ("nur ein <Operand> fr BOZ-Befehl zugelassen");

            }

            if (line.operandA.addressMode!=Operand.MODE_DIRECT) {

                throw new CompilerException ("Nur direkte Adressierung fr BOZ-Befehl zugelassen");

            }

            if (!line.offsetFound) throw new CompilerException

                ("<Offset> fr BOZ-Befehl erforderlich");



            objectCode[pc]=BOZ;

            pc++;

            objectCode[pc]=line.offset;

            pc++;

            addObjectCode (line);



        }





        if (line.operation==ParsedLine.OP_JMN) {

            if ((line.operandA==null) || (line.operandB==null)) {

                throw new CompilerException ("zwei Operanden fr JMN-Befehl erforderlich");

            }

            if ((line.operandA.addressMode!=Operand.MODE_DIRECT) ||

               (line.operandB.addressMode!=Operand.MODE_DIRECT)) {

                throw new CompilerException ("Nur direkte Adressierungen fr JMN-Befehl zugelassen");

            }

            if (line.offsetFound) throw new CompilerException

                ("<Offset> fr JMN-Befehl unzulssig");



            objectCode[pc]=JMN;

            pc++;

            objectCode [pc]=0;              //offset

            pc++;

            addObjectCode (line);



        }



        if (line.operation==ParsedLine.OP_JMP) {

            if (line.operandB!=null)  {

                throw new CompilerException ("nur ein <Operand> fr JMP-Befehl zugelassen");

            }

            if (line.offsetFound) {

                throw new CompilerException ("<Offset> fr JMP-Befehl unzulssig");

            }

            if (line.operandA.addressMode==Operand.MODE_CONSTANT) {

                throw new CompilerException ("Keine konstanten Adressierungen fr JMP-Befehl zugelassen");

            }



            if (line.operandA.addressMode==Operand.MODE_DIRECT) {

                objectCode[pc]=JMP;

                pc++;

            }



            if (line.operandA.addressMode==Operand.MODE_INDIRECT) {

                objectCode[pc]=JMPatA;

                pc++;

            }

	    objectCode [pc]=0;        //offset

	    pc++;

            addObjectCode (line);



        }



        if (line.operation==ParsedLine.OP_JMZ) {

            if ((line.operandA==null) || (line.operandB==null)) {

                throw new CompilerException ("zwei Operanden fr JMZ-Befehl erforderlich");

            }

            if ((line.operandA.addressMode!=Operand.MODE_DIRECT) ||

               (line.operandB.addressMode!=Operand.MODE_DIRECT)) {

                throw new CompilerException ("Nur direkte Adressierungen fr JMZ-Befehl zugelassen");

            }

            if (line.offsetFound) throw new CompilerException

                ("<Offset> fr JMZ-Befehl unzulssig");



            objectCode[pc]=JMZ;

            pc++;

            objectCode [pc]=0;              //offset

            pc++;

            addObjectCode (line);



        }



        if (line.operation==ParsedLine.OP_MMR) {

            if ((line.operandA==null) || (line.operandB==null)) {

                throw new CompilerException ("zwei Operanden fr MMR-Befehl erforderlich");

            }

            if ((line.operandA.addressMode!=Operand.MODE_DIRECT) ||

               (line.operandB.addressMode!=Operand.MODE_DIRECT)) {

                throw new CompilerException ("Nur direkte Adressierungen fr MMR-Befehl zugelassen");

            }

            if (!line.offsetFound) throw new CompilerException

                ("<Offset> fr MMR-Befehl erforderlich");



            objectCode[pc]=MMR;

            pc++;

            objectCode [pc]=line.offset;              //offset

            pc++;

            addObjectCode (line);



        }



        if (line.operation==ParsedLine.OP_MOH) {

            if ((line.operandA==null) || (line.operandB==null)) {

                throw new CompilerException ("zwei Operanden fr MOH-Befehl erforderlich");

            }

            if ((line.operandA.addressMode!=Operand.MODE_DIRECT) ||

               (line.operandB.addressMode!=Operand.MODE_DIRECT)) {

                throw new CompilerException ("Nur direkte Adressierungen fr MOH-Befehl zugelassen");

            }

            if (!line.offsetFound) throw new CompilerException

                ("<Offset> fr MOH-Befehl erforderlich");



            objectCode[pc]=MOH;

            pc++;

            objectCode [pc]=line.offset;              //offset

            pc++;

            addObjectCode (line);



        }



        if (line.operation==ParsedLine.OP_MOV) {

            if ((line.operandA==null) || (line.operandB==null)) {

                throw new CompilerException ("zwei Operanden fr MOV-Befehl erforderlich");

            }

            if (!line.offsetFound) throw new CompilerException

                ("<Offset> fr MOV-Befehl erforderlich");



            if (line.operandA.addressMode==Operand.MODE_DIRECT) {

                if (line.operandB.addressMode==Operand.MODE_CONSTANT) {

                    throw new CompilerException ("ungltige Adressierung des zweiten Operanden beim MOV-Befehl");

                }

                if (line.operandB.addressMode==Operand.MODE_DIRECT) {

                    objectCode[pc]=MOV;

                }

                if (line.operandB.addressMode==Operand.MODE_INDIRECT) {

                    objectCode[pc]=MOVatB;

                }

            }



            if (line.operandA.addressMode==Operand.MODE_CONSTANT) {

                if (line.operandB.addressMode!=Operand.MODE_DIRECT) {

                    throw new CompilerException ("ungltige Adressierung des zweiten Operanden beim MOV-Befehl");

                }

                objectCode[pc]=MOVhashA;

            }



            if (line.operandA.addressMode==Operand.MODE_INDIRECT) {

                if (line.operandB.addressMode!=Operand.MODE_DIRECT) {

                    throw new CompilerException ("ungltige Adressierung des zweiten Operanden beim MOV-Befehl");

                }

                objectCode[pc]=MOVatA;

            }



            pc++;

            objectCode [pc]=line.offset;              //offset

            pc++;

            addObjectCode (line);



        }



        if (line.operation==ParsedLine.OP_MUL) {

            if ((line.operandA==null) || (line.operandB==null)) {

                throw new CompilerException ("zwei Operanden fr MUL-Befehl erforderlich");

            }

            if ((line.operandA.addressMode!=Operand.MODE_DIRECT) ||

               (line.operandB.addressMode!=Operand.MODE_DIRECT)) {

                throw new CompilerException ("Nur direkte Adressierungen fr MUL-Befehl zugelassen");

            }

            if (line.offsetFound) throw new CompilerException

                ("<Offset> fr MUL-Befehl unzulssig");



            objectCode[pc]=MUL;

            pc++;

            objectCode [pc]=0;              //offset

            pc++;

            addObjectCode (line);



        }



        if (line.operation==ParsedLine.OP_NEG) {



            if (line.operandB!=null) {

                throw new CompilerException ("nur ein <Operand> fr NEG-Befehl zugelassen");

            }

            if (line.operandA.addressMode!=Operand.MODE_DIRECT) {

                throw new CompilerException ("Nur direkte Adressierung fr NEG-Befehl zugelassen");

            }

            if (line.offsetFound) throw new CompilerException

                ("<Offset> fr NEG-Befehl unzulssig");



            objectCode[pc]=NEG;

            pc++;

            objectCode[pc]=0; //offset

            pc++;

            addObjectCode (line);



        }



        if (line.operation==ParsedLine.OP_SIC) {



            if (line.operandB!=null) {

                throw new CompilerException ("nur ein <Operand> fr SIC-Befehl zugelassen");

            }

            if (line.operandA.addressMode!=Operand.MODE_DIRECT) {

                throw new CompilerException ("Nur direkte Adressierung fr SIC-Befehl zugelassen");

            }

            if (line.offsetFound) throw new CompilerException

                ("<Offset> fr SIC-Befehl unzulssig");



            objectCode[pc]=SIC;

            pc++;

            objectCode[pc]=0; //offset

            pc++;

            addObjectCode (line);



        }



        if (line.operation==ParsedLine.OP_SLL) {



            if (line.operandB!=null) {

                throw new CompilerException ("nur ein <Operand> fr SLL-Befehl zugelassen");

            }

            if (line.operandA.addressMode!=Operand.MODE_DIRECT) {

                throw new CompilerException ("Nur direkte Adressierung fr SLL-Befehl zugelassen");

            }

            if (!line.offsetFound) throw new CompilerException

                ("<Offset> fr SLL-Befehl erforderlich");



            objectCode[pc]=SLL;

            pc++;

            objectCode[pc]=line.offset;

            pc++;

            addObjectCode (line);



        }



        if (line.operation==ParsedLine.OP_SRL) {



            if (line.operandB!=null) {

                throw new CompilerException ("nur ein <Operand> fr SRL-Befehl zugelassen");

            }

            if (line.operandA.addressMode!=Operand.MODE_DIRECT) {

                throw new CompilerException ("Nur direkte Adressierung fr SRL-Befehl zugelassen");

            }

            if (!line.offsetFound) throw new CompilerException

                ("<Offset> fr SRL-Befehl erforderlich");



            objectCode[pc]=SRL;

            pc++;

            objectCode[pc]=line.offset;

            pc++;

            addObjectCode (line);



        }



        if (line.operation==ParsedLine.OP_SUB) {

            if ((line.operandA==null) || (line.operandB==null)) {

                throw new CompilerException ("zwei Operanden fr SUB-Befehl erforderlich");

            }

            if ((line.operandA.addressMode!=Operand.MODE_DIRECT) ||

               (line.operandB.addressMode!=Operand.MODE_DIRECT)) {

                throw new CompilerException ("Nur direkte Adressierungen fr SUB-Befehl zugelassen");

            }

            if (line.offsetFound) throw new CompilerException

                ("<Offset> fr SUB-Befehl unzulssig");



            objectCode[pc]=SUB;

            pc++;

            objectCode [pc]=0;              //offset

            pc++;

            addObjectCode (line);



        }





        if (line.operation==ParsedLine.OP_XOR) {

            if ((line.operandA==null) || (line.operandB==null)) {

                throw new CompilerException ("zwei Operanden fr XOR-Befehl erforderlich");

            }

            if ((line.operandA.addressMode!=Operand.MODE_DIRECT) ||

               (line.operandB.addressMode!=Operand.MODE_DIRECT)) {

                throw new CompilerException ("Nur direkte Adressierungen fr XOR-Befehl zugelassen");

            }

            if (line.offsetFound) throw new CompilerException

                ("<Offset> fr XOR-Befehl unzulssig");



            objectCode[pc]=XOR;

            pc++;

            objectCode [pc]=0;              //offset

            pc++;

            addObjectCode (line);



        }



        if (line.operation==ParsedLine.OP_VALUE) {

            addint (line.value);

        }

    }





    /** Schreibt den Objektcode und das Reloc-Array in das CoreInfo Objekt.

    

      * Sind noch nicht alle Vorwrtsreferenzen auf Label aufgelst, wird

      

      * eine CompilerException erzeugt.

    

      */

      

      

    public void writeCode (CoreInfo coreInfo) throws CompilerException{

        coreInfo.size=pc;

        coreInfo.core=new byte [pc];

        LabelInfo li;

        String s;

        short value;

        short relocpc=0;





        // berprfung, ob alle Label-Referenzen gltig sind
        for (int i=0; i<unresolved.size(); i++) {

            li=(LabelInfo) unresolved.elementAt(i);

            s=li.name;

            if (relocpc>509) {

                throw new CompilerException ("Maximale Anzahl an Labels berschritten");

            }



            if (labels.containsKey(s)) {

                value=((Short) labels.get (s)).shortValue();

            }

            else throw new CompilerException ("Referenz auf nicht vorhandenes Label: " + s);



        // Erstellen des RelocArray
            reloc[relocpc]=li.position; 

            relocpc++;

            reloc[relocpc]=value+li.distance;

            relocpc++;



        }



        // Kopieren des Objektcodes in das CoreInfo-Objekt
        for (int i=0; i<pc; i++) {

            coreInfo.core [i]=objectCode[i];

        }



        //Kopieren des Reloc-Arrays in das CoreInfo Object


	coreInfo.reloc=new int [relocpc];
        for (int i=0; i<relocpc; i++) {
	    coreInfo.reloc [i]=reloc[i];
        }


        coreInfo.alive=true;
    }








/** Die Methode prft, ob in dieser Zeile ein Label fr einen der Operanden



  * eingesetzt wurde. In diesem Fall wird ein Verweis darauf in den Vector

  

  * unresolved geschrieben. Dies ist erforderlich, um daraus spter die Informationen 
  

  * ber das Reloc-Array zu rekonstruieren und zu berprfen, ob

  

  * es sich um ein gltiges Label handelt.

  

  * Wurde kein Label eingesetzt, wird der normale ObjectCode angefgt.

*/



    private void addObjectCode (ParsedLine p) {

        LabelInfo labelInfoA=new LabelInfo();

        LabelInfo labelInfoB=new LabelInfo();



        if (p.operandA.label!=null) {

            labelInfoA.name=p.operandA.label;

            labelInfoA.distance=p.operandA.distance;

            labelInfoA.position=pc;

            unresolved.addElement (labelInfoA);

            pc+=2;

        }

        else {

            addint (p.operandA.address);

        }



        if (p.operandB!=null) {

            if (p.operandB.label!=null) {

                labelInfoB.name=p.operandB.label;

                labelInfoB.distance=p.operandB.distance;

                labelInfoB.position=pc;

                unresolved.addElement (labelInfoB);

                pc+=2;

            }

            else {

                addint (p.operandB.address);

            }

        }

    }







    /** Zerlegt einen int in zwei bytes und fgt diese dem Objektcode hinzu */

    

    private void addint (int i) {

        byte lsb=(byte) i;

        byte msb=(byte) (i>>>8);

        objectCode[pc]=lsb;

        pc++;

        objectCode[pc]=msb;

        pc++;

    }



}

