/*

 * @(#)Scanner.java    20.06.1999

 */


/*
 * Diese Klasse fhrt die lexikalische Analyse des Quelltextes durch. Der Quelltext

 * wird aus einem ImputStream gelesen und zeilenweise analysiert.

 * @author Benjamin Stehr


 */




package corewar.compiler;

import corewar.CoreInfo;
import java.io.*;
import java.util.*;

public class Scanner {
    BufferedReader buffer;
    int LineNumber;
    CoreInfo Core;

    String tokenLine;







    /** Erzeugt ein Scanner-Objekt mit inStream als Datenquelle. */



    public Scanner (InputStream inStream, CoreInfo coreinfo) {

        InputStreamReader in=new InputStreamReader (inStream);

        buffer=new BufferedReader (in);

        int LineNumber=1;

        Core=coreinfo;

        tokenLine=null;

    }











    /** Prft, ob noch weitere Quelltextzeilen vorhanden sind.


     * Falls ber inStream noch eine weitere Quelltextzeile geliefert wird, 

     * speichert diese Methode diese Zeile in tokenLine. Auerdem wird 

     * die aktuelle Zeilennummer um 1 erhht.

     * Sollte es sich bei der gerade gelesenen Zeile allerdings um eine reine

     * Kommentarzeile handeln, wird nach der nchsten Zeile gesucht, die 

     * nicht nur aus Kommentaren besteht.

     * Wurde eine solche Zeile gefunden, wird true zurckgeliefert.



     */



    public boolean hasMoreLines() throws IOException{

        if (buffer.ready()) {

             tokenLine=buffer.readLine();

             LineNumber++;

                                        

             while (isCommentLine (tokenLine)) {

                 if (buffer.ready()) {

                     tokenLine=buffer.readLine();

                     LineNumber++;

                 }

                 else {return false; }

             }

             return true;

        }

        return false;

    }





    /** Liefert die nchste Zeile des Quelltextes in Form eines Vektors aus 

     * Token-Objekten. Durch den Aufruf von hasMoreLines() liegt diese 

     * Quelltextzeile bereits als String in tokenLine vor.

     * Diese Methode bedient sich nun des StringTokenizer, um eine Aufteilung

     * dieser Zeile in einzelne Token  zu erreichen. Da ein Token-Objekt nicht nur

     * den eigenltichen Inhalt enthlt, sondern auch eine Variable, die angibt worum

     * es sich bei diesem String handelt, wird deren Wert durch die Methode 

     * whichToken() bestimmt.

     * Die einzelnen Token-Objekte werden dem Vektor v hinzugefgt, bis die

     * komplette Zeile bearbeitet wurde.

     * Anschlieend wird dieser Vektor zurckgeliefert.

     */



    public Vector  nextLine() {



        Vector v=new Vector ();

        String line=null;

        String stringTokenizerResult=null;



        line=tokenLine;

        StringTokenizer st=new StringTokenizer (line, "() ,'/@+-:;#", true);

            while (st.hasMoreTokens()) {

                stringTokenizerResult=st.nextToken();
                
                // Diese Abfrage ist erforderlich um die Tokens, welche nur
                // aus Leerzeichen bestehen auszufiltern
                
                if (stringTokenizerResult.startsWith(" ")==false) {

                    v.addElement (new Token (whichToken(stringTokenizerResult) ,stringTokenizerResult));

                }

            }

        

        return (v);

    }













    /** Liefert die Zeilennummer der gerade analysierten Quelltextzeile zurck*/



    public int getLineNumber() {

        return LineNumber;

    }











    /** Fhrt die eigentliche lexikalische Analyse duch und liefert eine Konstante 

     * zurck, welche den Typ des bergebene Strings darstellt. Sie unterschiedet

     * zwischen Zahlen, den einzelnen Steuerzeichen und Strings, die zu keiner

     * dieser beiden Kategorien zhlen. Die jeweiligen Konstanten wurden in der Klasse

     * Token definiert.


     */



    private int whichToken (String s) {



        boolean b=true;

        for (int i=0; i<s.length(); i++) {

            if (!Character.isDigit(s.charAt( i ))) { b=false; }

        }

        if (b) {return Token.NUMBER; }

        if (s.equals("#")) {return Token.HASH; }

        if (s.equals("@")) {return Token.ATSIGN; }

        if (s.equals(":")) {return Token.COLON; }

        if (s.equals(",")) {return Token.COMMA; }

        if (s.equals("(")) {return Token.BRACE_OPEN; }

        if (s.equals(")")) {return Token.BRACE_CLOSE; }

        if (s.equals("+")) {return Token.PLUS; }

        if (s.equals("-")) {return Token.MINUS; }

        return Token.STRING;

    }











    /**berprft, ob es sich bei dem bergenem String um eine reine Kommentarzeile
    
     * handelt. Falls sich in solch einer Zeile Informationen ber Autor und Name
     
     * des Cores befinden, werden diese in coreInfo gespeichert.

     * Der Name des Autoren/Cores wird in der Form ";author  " bzw. ";name "
     
     * im Quelltext angegeben. Der Rest der Zeile wird dann als Name betrachtet.


     */



    private boolean isCommentLine (String s) {

        int i=0;



        if (s.length()==0) return true;



        while  ((s.charAt(i)==' ') && (i+1<s.length() )) {

            i++;

        }

        if (s.charAt (i)==' ') return true;
        if (s.charAt (i)==';') {

            try {

                if (s.substring(i+1,i+8).equals ("author ")) {

                    Core.author=s.substring(i+8, s.length());

                }

                if (s.substring(i+1,i+6).equals ("name ")) {

                    Core.coreName=s.substring(i+6, s.length());

                }

            }

            catch (StringIndexOutOfBoundsException E) {}

            return true;

        }

        return false;

    }
    
}

