/*
 * @(#)MetaClientSocket.java    19.6.1999
 */


package corewar.metaserver;

import corewar.NetworkSocket;
import corewar.CoreInfo;
import corewar.ServerInfo;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.net.Socket;

/**
 * Die Klasse MetaClientSocket stellt eine Netzwerkverbindung zu einem Client
 * dar und ermglicht das stndige Warten auf Nachrichten mit Hilfe eines
 * eigenen Threads.
 *
 * @author  Ivo Kasiuk
 */
public class MetaClientSocket extends NetworkSocket {

    /* Die Client-Identifikation. */
    private static final byte[] clientID = { I_AM_CLIENT };

    /* Der MetaServer, an den die Nachrichten vom Client weitergleitet werden */
    private MetaServer metaServer;
    
    /* Das vom Client gesendete CoreInfo-Objekt. */
    private CoreInfo coreInfo = null;
    
    /**
     * Prft die Anmeldung des Clients und empfngt bei erfolgreicher Anmeldung
     * das CoreInfo-Objekt von ihm.
     * @param metaServer  Nachrichten vom Client werden an die entsprechenden
     *                    Methoden dieses MetaServers weitergeleitet.
     * @param client      Der sich anmeldende Client.
     * @exception IOException  wenn bei der Anmeldung des Clients ein Fehler
     *                         auftritt.
     */
    public MetaClientSocket(MetaServer metaServer, Socket client)
            throws IOException {
        super(client, clientID);
        this.metaServer = metaServer;

        ObjectInputStream inStream = getInStream();
        byte              msgID    = inStream.readByte();
        if (msgID != CORE_INFO) {
            throw new IOException((msgID == BYE)
                                  ? "Verbindung vom Client abgebrochen"
                                  : "Fehler bei der Anmeldung");
        }
        try {
            coreInfo = (CoreInfo) inStream.readObject();
        } catch (Exception e) {
            throw new IOException("Fehler beim Lesen der Coreinformationen");
        }
    }

    /**
     * Liefert das vom Client gesendete CoreInfo-Objekt.
     */
    public CoreInfo getCoreInfo() {
        return coreInfo;
    }

    /**
     * Benachrichtigt den Client, da ein Server fr ihn bereit ist und beendet
     * anschlieend die Verbindung.
     * @param serverInfo  Die Informationen ber den Server.
     * @exception IOException  wenn bei der Kommunikation ein Fehler auftritt.
     */
    public void thisIsYourServer(ServerInfo serverInfo) {
        ObjectOutputStream outStream = getOutStream();
        try {
            outStream.writeByte(THIS_IS_YOUR_SERVER);
            outStream.writeObject(serverInfo);
            outStream.flush();
            getSocket().close();
        } catch (IOException ioe) {}
    }
    
    /**
     * Meldet den Client beim MetaServer ab, wenn die Verbindung abbricht oder
     * vom Client beendet wird.
     * @param msgID  Die Identifikation der Nachricht.
     * @return  true, wenn die Nachrichtenverarbeitung fortgesetzt werden soll.
     * @exception IOException  wenn bei der Kommunikation ein Fehler auftritt.
     */
    protected boolean handleMessage(byte msgID) throws IOException {
        metaServer.removeClient(this);
        try {
            getSocket().close();
        } catch (IOException ioe) {}
        return false;
    }
}
