/*
 * @(#)MetaServerSSocket.java    23.6.1999
 */
package corewar.metaserver;

import corewar.CWProtocol;
import corewar.MessageBox;
import corewar.NetworkSSocket;
import java.net.Socket;
import java.io.IOException;

/**
 * Die Klasse MetaServerSSocket nimmt die sich anmeldenden Server entgegen.
 *
 * @author  Stephan Keitz
 */
public class MetaServerSSocket extends NetworkSSocket implements CWProtocol{
  
    /** Referenz auf den MetaServer */
    private MetaServer metaServer;

    /** 
     * Richtet einen ServerSocket ein, an den sich Server anmelden knnen. Wenn
     * 'locked' den Wert 'true' hat, ist der Socket gesperrt, d.h. alle
     * Verbindungen werden abgelehnt.
     * @param ms      Die Referenz auf die Klasse MetaServer
     * @param locked  true, wenn der ServerSocket gesperrt ist
     * @exception IException  falls der ServerSocket nicht erstellt werden kann
     */
    public MetaServerSSocket(MetaServer ms, boolean locked) throws IOException {
      super(DEFAULT_META_SERVER_PORT, locked);
      metaServer = ms;
    }

    /**
     * Wenn sich jemand anmeldet wird ueberprueft, ob es sich hierbei um einen
     * Server handelt. Im positiven Falle wird ein neuer MetaServerSocket erstellt.
     * @param socket  Die Verbindung zu einem Server
     */
    protected void handleConnection(Socket socket) {
      byte wantedID[] = { I_AM_SERVER };
      MetaServerSocket server = null;
      try {
        server = new MetaServerSocket(socket, wantedID, metaServer);
        server.start();
        metaServer.addServer(server);
      } catch (IOException e) {
        MessageBox mb = new MessageBox(
          "Server annahme fehlgeschlagen !\n"+e.getMessage(),
          "MetaServerSSocket Fehler !", metaServer, MessageBox.OK);
        int result = mb.execute();
        try { (server.getSocket()).close(); }
        catch (IOException e2) {}
      }
    }

}