/*
 * @(#)MetaServerSocket.java    24.6.1999
 */

package	corewar.metaserver;

import corewar.MessageBox;
import corewar.NetworkSocket;
import corewar.ParamInfo;
import corewar.ServerInfo;

import java.io.IOException;
import java.io.ObjectOutputStream;

import java.lang.ClassNotFoundException;

import java.net.Socket;

import java.util.Vector;

/**
 * Die Klasse MetaServerSocket stellt eine Netzwerkverbindung zu einem Server dar
 * und ermoeglicht das staendige Warten auf Nachrichten mit Hilfe eines eigenen
 * Threads. Auuserdem werden neu ankommende und wartende Clients beim Server
 * angemeldet und dann losgeschickt.
 *
 * @author  Stephan Keitz
 */
public class MetaServerSocket extends NetworkSocket {

  /** Referenz auf den MetaServer */
  private MetaServer metaServer	= null;
  /** Die Paraminfo, die spaeter von jedem Server geschickt wird */
  private ParamInfo paramInfo =	null;
  /** Die Anzahl der Clients, die der Server noch braucht */
  private int coresNeeded = 0;
  /** Buffer mit den zu meldenden Clients */
  private Vector clients = new Vector();
  /** Die zu den Clients gehoerenden Identifikationsnummern */
  private Vector ids = new Vector();
  /** Die Portnummer des Servers */
  private int portNumber = DEFAULT_SERVER_PORT;

  /**
   * Prueft die Anmeldung des Servers. Bei einem auftretenden Fehler wird eine
   * IOException ausgeloest.
   * @param server           Der Socket mit der verbindung zum Server
   * @param wantedID[]       Array mit den Meldungen , die erwartet werden
   * @param ms               Die Referenz auf den MetaServer
   * @exception IOException  Falls der MetaServerSocket nicht erstellt werden kann
   */
  public MetaServerSocket(Socket server, byte[]	wantedID, MetaServer ms) throws	IOException {
    super(server, wantedID);
    metaServer = ms;
  }

  /**
   * Liefert die zu diesem Server gehoerenden Parameter.
   * @return  Die Parameter
   */
  public ParamInfo getParamInfo() {
    return paramInfo;
  }

  /**
   * Liefert die Anzahl der vom Server noch gebrauchten Cores zurueck.
   * @return  Anzahl der gebrauchten Cores
   */
  public int getNumCoresNeeded() {
    return coresNeeded;
  }

  /**
   * Meldet dem Server welcher Client als naechstes kommt.
   * @param client           Der Client
   * @param connectID        Identifikationsnummer des Clients
   * @exception IOException  falls die Datenuebertragung nicht funktioniert
   */
  public void thisIsYourClient(MetaClientSocket	client,	byte connectID)	throws IOException {
    clients.addElement(client);
    ids.addElement(new Byte(connectID));
    ObjectOutputStream out = getOutStream();
    out.writeByte(THIS_IS_YOUR_CLIENT);
    out.writeByte(connectID);
    out.flush();
    coresNeeded--;
  }

  /**
   * Reagiert auf die ankommenden Meldungen.
   * @param msgID            Die Meldung
   * @return                 true, wenn weitergemacht werden soll
   * @exception IOException  falls die Meldung unbekannt ist
   */
  protected boolean handleMessage(byte msgID) throws IOException {
    switch (msgID) {
      case BYE:
      case CONNECTION_ERROR:
        (getSocket()).close();
        metaServer.removeServer(this);
        return false;
      case PARAM_INFO:
        try {
          paramInfo = (ParamInfo)(getInStream()).readObject();
          portNumber = (getInStream()).readInt();
          coresNeeded = paramInfo.getCoreNumber();
          metaServer.distributeServer(this);
        } catch  (ClassNotFoundException cnfe) {
          MessageBox mb	= new MessageBox(
            "Die Klasse ParamInfo fehlt !\n" + cnfe.getMessage(),
            "MetaServerSocket Fehler !", metaServer, MessageBox.OK);
          int result = mb.execute();
          abortConnection();
        } catch (IOException ioe) {
          MessageBox mb = new MessageBox(
            "PARAM_INFO konnte nicht gelesen werden !\n" + ioe.getMessage(),
            "MetaServerSocket Fehler !", metaServer, MessageBox.OK);
          int result = mb.execute();
          throw ioe;
        }
        return true;
      case CORES_NEEDED:
        try {
          coresNeeded =	(getInStream()).readByte();
          metaServer.distributeServer(this);
        } catch	(IOException ioe) {
          MessageBox mb	= new MessageBox(
            "CORES_NEEDED konnte nicht gelesen werden !\n" + ioe.getMessage(),
            "MetaServerSocket Fehler !", metaServer, MessageBox.OK);
          int result = mb.execute();
          throw ioe;
        }
        return true;
      case READY_FOR_CLIENT:
        Socket socket = getSocket();
        MetaClientSocket client = (MetaClientSocket) clients.firstElement();
        byte connectID = ((Byte) ids.firstElement()).byteValue();
        ServerInfo serverInfo = new ServerInfo(socket.getInetAddress(), portNumber, connectID);
        client.thisIsYourServer(serverInfo);
        clients.removeElement(client);
        ids.removeElement(new Byte(connectID));
        return true;
      default:
        throw new IOException("Unbekannte Nachricht");
    }
  }

}