/*
 * @(#)ClientAccepter.java    17.6.1999
 */
package	corewar.server;

import corewar.CWProtocol;
import corewar.ParamInfo;
import corewar.CoreInfo;
import corewar.MessageBox;
import java.awt.Frame;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.ObjectInputStream;
import java.lang.Thread;
import java.net.Socket;

/**
 * Die Klasse ClientAccepter ueberprueft, ob der Client	ein Client ist und ob
 * sein Core den eingestellten Parametern entspricht bzw. wenn er vom MetaServer
 * angemeldet wurde, ob er der erwartete Client ist.
 *
 * @author  Stephan Keitz
 */
public class ClientAccepter extends Thread implements CWProtocol {

  /** Referenz auf den NetManager */
  private NetManager netManager;
  /** Der vom ServerClientSSocket uebergebene Socket */
  private Socket client;

  /**
   * Der Konstruktor merkt sich	die Referenz auf den NetManager und den Socket,
   * der vom ServerClientSSocket uebergeben wurde, und startet die Ueberpruefung.
   */
  public ClientAccepter(NetManager nm, Socket socket) {
    netManager = nm;
    client = socket;
    start();
  }

  /**
   * Die Routine baut einen Datenstrom zum Client auf, erfragt Daten
   * von den Clients und stellt sie somit den anderen Methoden zur Verfuegung.
   */
  public void run() {
    try	{
      // Ist der Client	ein Client und ist er erwuenscht ?
      if (!correct()) sendBye();
      else {
        ServerClientSocket scs = new ServerClientSocket(client);
        ObjectInputStream in = scs.getInStream();
        CoreInfo ci;
        // Passen seine	Werte zu den Parametern	?
        if (in.readByte() == CORE_INFO)	{
          ci = (CoreInfo) in.readObject();
        } else throw new IOException("CORE_INFO	erwartet !");
        // Wenn	alles passt Werte und Client weitergeben
        if (parametersFit(ci)) {
          netManager.addClient(scs,ci);
        }
      }
    }
    // Fehlerbehandlungen
    catch (IOException e) {
      MessageBox mb = new MessageBox(
        "Client	auf Korrektheit	pruefen	fehlgeschlagen !\n"+e.getMessage(),
        "ClientAccepter	Fehler !", new Frame(),	MessageBox.OK);
      int result = mb.execute();
      try { client.close(); }
      catch (IOException e2) {}
    }
    catch (ClassNotFoundException e) {
      MessageBox mb = new MessageBox(
      e.getMessage(),	"ClientAccepter	Fehler !", new Frame(),	MessageBox.OK);
      int result = mb.execute();
      try { client.close(); }
      catch (IOException e2) {}
    }
  }

  /**
   * Ueberprueft seine Identifikationsnummer, ob auf der anderen Seite ein Client ist
   * und ob er noch gebraucht wird.
   * @return  true, wenn der Client ein	Client und erwuenscht ist
   */
  private boolean correct() throws IOException {
    InputStream	in  = client.getInputStream();
    boolean allright = true;
    //Pruefen ob an der	anderen	Seite ein Client ist
    byte id = (byte) in.read();
    if (id != I_AM_CLIENT) allright = false;
    //Pruefen ob noch clients gebraucht werden
    allright = netManager.getCoresNeeded() > 0;
    //Pruefen ob dieser Client erwuenscht ist
    if (allright) {      
      id = (byte) in.read();
      if (!netManager.getIndependentMode()) allright = netManager.deleteIDifpossible(id);
    }
    return allright;
  }

  /**
   * Ueberprueft, ob die Groesse des Cores stimmt.
   * @return  true, wenn die Groesse passt
   */
  private boolean parametersFit(CoreInfo ci) {
    ParamInfo pi = netManager.getParamInfo();
    return (ci.size <= pi.getMaxCoreSize());
  }

  /**
   * "Verabschiedet" den Client, wenn er nicht erwuenscht ist.
   * @exception	IOException  wenn ein Fehler auftritt
   */
  private void sendBye() throws	IOException {
    OutputStream out = client.getOutputStream();
    out.write(BYE);
    out.flush();
    client.close();
  }

}
