/*
 * @(#)ClientCommunicator.java	 15.6.1999
 */
package	corewar.server;

import corewar.CWProtocol;
import java.io.IOException;
import java.lang.System;
import java.util.Enumeration;
import java.util.Vector;

/**
 * Die Klasse ClientCommunicator verwaltet die Clients in einer Liste
 * und steuert die Kommunikation mit ihnen.
 *
 * @author  Stephan Keitz
 */
public class ClientCommunicator	implements CWProtocol {

  /** Die Liste	mit den	Sockets	des Clients */
  private Vector clients = new Vector();
  /** Die Anzahl der Clients, die bereit sind Daten zu empfangen */
  private byte clientsReady = 0;
  /** Referenz auf den NetManager */
  private NetManager netManager;

  /**
   * Der Konstruktor initialisiert die Referenz	zum NetManager.
   * @param nm	Eine Referenz zum NetManager
   */
  public ClientCommunicator(NetManager nm) {
    netManager = nm;
  }

  /**
   * Fuegt ServerClientSockets in die Liste ein, verteilt Nummern fuer
   * die Clients und startet den Thread, der die Sockets abhoert.
   * @param scs	 Referenz auf den einzufuegenden ServerClientSocket
   */
  public synchronized void addClient(ServerClientSocket scs)	{
    byte coresNeeded = netManager.getCoresNeeded();
    scs.setClientCommunicator(this, (byte) (netManager.getCoreInfoVectorSize() - 1));
    clients.addElement(scs);
    scs.start();
    sendData(WAIT_FOR_CORES,new	Byte(coresNeeded));
    if (coresNeeded <= 0) netManager.startTournament();
  }

  /**
   * Schliesst die Verbindung zu allen Clients und entfernt sie aus der Liste.
   */
  public void closeClients() {
    clientsReady = 0;
    ServerClientSocket scs;
    System.out.println("- ClientCommunicator.closeClients()");
    while (!clients.isEmpty()) {
      scs = (ServerClientSocket) clients.firstElement();
      try {
        scs.abortConnection();
      }
      catch (IOException e) {
        try { (scs.getSocket()).close(); }
        catch (IOException e2) {}
      }
      clients.removeElement(scs);
    }
  }

  /**
   * Routine, damit sich die ServerClientSockets selbst	beenden koennen.
   * Der Thread des Sockets wird angehalten und die Referenz auf den
   * ServerClientSocket	wird aus der Liste geloescht.
   * @param scs  Eine Referenz zu einem ServerClientSocket
   */
  public void killClient(ServerClientSocket scs) {
    System.out.println("- ClientCommunicator.killClient()");
    clients.removeElement(scs);
    if (clients.size() == 0) noMoreClients();
  }

  /**
   * Hier wird gewartet, bis alle Clients fertig sind zum Daten empfangen,
   * wenn sie nicht zu langsam sind.
   */
  public void waitForClientsReady() {
    System.out.println("- ClientCommunicator.waitForClientsReady()");
    System sys = null;
    long TIME_OUT = 20000;
    long timeOut = sys.currentTimeMillis() + TIME_OUT;
    while ((clientsReady < clients.size()) && (timeOut >= sys.currentTimeMillis())) {
      if (timeOut < sys.currentTimeMillis()) System.out.println("- ClientCommunicator.waitForClientsReady() : time out");
    }
  }

  /**
   * Routine fuer die ServerClientSockets um zu	melden,	dass ihr Client
   * bereit ist, um Daten zu empfangen.
   */
  public void incrementClientReady() {
    clientsReady++;
    System.out.println("- ClientCommunicator.incrementClientReady() -> " + clientsReady + " of " + clients.size());
  }

  /**
   * Routine zum Verschickt von	Nachrichten und	Daten an alle Clients.
   * Ggf. werden die Daten noch aufbereitet.
   * @param id	  Typ der Nachricht
   * @param data  Daten	fuer die Nachricht
   */
  public void sendData(byte id,	Object data) {    
    Enumeration enum = clients.elements();
    ServerClientSocket scs;
    if (id == END_OF_ROUND) clientsReady = 0;
    if (id == RAM_INFO) {
      RamInfo ri = (RamInfo) data;
      data = ri.addr;
    }
    while(enum.hasMoreElements()) {
      scs = (ServerClientSocket) enum.nextElement();
      try { scs.sendData(id, data); }
      catch (IOException e) {}
    }
    if (id == END_OF_ROUND) waitForClientsReady();
  }

  /**
   * Mitteilung	an den NetManager, dass	kein Client mehr vorhanden ist.
   */
  private void noMoreClients() {
    System.out.println("- ClientCommunicator.noMoreClients()");
    netManager.noMoreClients();
  }

}
