/*
 * @(#)ListWatcher.java    9.6.1999
 */

package corewar.server;

import corewar.CWProtocol;
import corewar.MessageBox;
import java.awt.Frame;
import java.io.IOException;
import java.lang.System;
import java.util.Vector;

/**
 * Die Klasse ListWatcher merkt sich die vom Metaserver angemeldeten Clients
 * und schmeisst diese, wenn sie sich nicht innerhalb einer bestimmten Zeit
 * angemeldet haben, aus der Liste.
 *
 * @author  Stephan Keitz
 */
public class ListWatcher extends Thread implements CWProtocol {

  /** Anzahl der Cores, die maximal verwaltet werden */
  private final byte cores = 20;
  /** Vector mit den IDs, mit denen sich die Clients anmelden */
  private Vector ids = new Vector(cores);
  /** Vector mit den Zeiten, wann die Clients wieder aus der Liste entfernt werden */
  private Vector times = new Vector(cores);
  /** Referenz auf den NetManager */
  private NetManager netManager;
	
  /**
   * Merkt sich die Refernez auf den NetManager
   * @param sms  Die Referenz auf den Netmanager
   */
  public ListWatcher(NetManager nm){
    netManager = nm;
  }
	
  /**
   * Die Routine, die vom Thread ausgefuehrt wird bis keine Clients mehr in der
   * Liste stehen. Alle Cores, die sich nach einer bestimmten Zeit nicht 
   * angemeldet haben werden entfernt und dem Metaserver wird einer Nachricht
   * geschickt, dass man wieder einen Client braucht. Sind alle ueberfaelligen
   * Clients aus der Liste entfernt, so wird berechnet, wie lange der Thread
   * schlafen kann, bis er wieder wen aus der Liste entfernen muss.
   */
  public void run() {
    System sys = null;
    long minWaitTime = SERVER_CLIENT_TIMEOUT,
         endTime = 0,
         waitTime = 0;
    while(!ids.isEmpty()) {
      // berfllige Cores lschen
      for (int coreNr = 0; coreNr < ids.size(); coreNr++) {
        endTime = ((Long)times.elementAt(coreNr)).longValue();
        waitTime = endTime - sys.currentTimeMillis();
        // wenn Client ueberfaellig
        if (waitTime <= 0) {
          int id = ((Integer)ids.elementAt(coreNr)).intValue();
          System.out.println("- ListWatcher.run() : time out");
          boolean deleted = deleteIDifpossible(id);
          try {
            netManager.sendData(CORES_NEEDED, new Byte(netManager.getCoresNeeded()));
          }
          catch(IOException e) {
            MessageBox mb = new MessageBox(
              "CORES_NEEDED Meldung fehlgeschlagen !\n"+e.getMessage(),
              "ListWatcher Fehler !", new Frame(), MessageBox.OK);
            int result = mb.execute();
          }
        }
      }
      // neue Schlafzeit berechnen
      for (int coreNr = 0; coreNr < ids.size(); coreNr++) {
        endTime = ((Long)times.elementAt(coreNr)).longValue();
        waitTime = endTime - sys.currentTimeMillis();
        if (waitTime < minWaitTime) minWaitTime = waitTime;
      }
      try {
        sleep(minWaitTime);
      }
      catch(InterruptedException e) {
        MessageBox mb = new MessageBox(
          "Thread ListWatcher wurde beim Warten unterbrochen !\n"+e.getMessage(),
          "ListWatcher Fehler !", new Frame(), MessageBox.OK);
        int result = mb.execute();
      }
    }
	}
	
  /**
   * Es wird ein Client in die Liste aufgenommen und eingetragen, bis wann
   * er sich angemeldet haben muss. Wenn gerade kein Client in der Liste ist
   * laeuft der Thread ggf. nicht und muss neu gestartet werden.
   * @param id  Identifikationsnummer die vom Metaserver uebergeben wurde und
   *            mit der sich der Client anmelden mu.
   */
  public synchronized void queueID(int id) {
    System sys = null;
    ids.addElement(new Integer(id));
    System.out.println("- ListWatcher.queueID(" + id + ")");
    times.addElement(new Long(sys.currentTimeMillis() + SERVER_CLIENT_TIMEOUT));
    if (!isAlive()) start();
  }
	
  /**
   * Versucht, die ID eines erwarteten Clients aus der Liste zu loeschen und
   * meldet zurueck, ob die ID in der Liste war.
   * @param id  Identifikationsnummer die vom Metaserver uebergeben wurde und
   *            mit der sich der Client anmelden mute.
   * @return  Rueckmeldung, ob der Client in der Liste war
   */
  public synchronized boolean deleteIDifpossible(int id) {
    int position = ids.indexOf(new Integer(id));
    if (position >= 0) {
      ids.removeElementAt(position);
      times.removeElementAt(position);
    }
    System.out.println("- ListWatcher.deleteIDifpossible(" + id + ") : " + (position >= 0));
    return (position >= 0);
  }
  
}
