/*
 * @(#)ServerClientSocket.java	 14.6.1999
 */
package	corewar.server;

import corewar.CWProtocol;
import corewar.GameSetupInfo;
import corewar.NetworkSocket;
import corewar.MessageBox;
import java.awt.Frame;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.net.Socket;

/**
 * Die Klasse ServerClientSocket wickelt die Kommunikation zwischen dem Server und
 * dem Client ab.
 *
 * @author  Stephan Keitz
 */
public class ServerClientSocket	extends	NetworkSocket implements CWProtocol {

  /** Referenz auf den ClientCommunicator */
  private ClientCommunicator clientCommunicator;
  /** Die Identifikationsnummer	der einzelnen Clients */
  byte clientNr;

  /**
   * Der Kunstruktor erstellt nur das Objekt und sendet dem Client CONNECT_OK.
   * @exception IOException  wenn bei der Kommunikation was schief geht
   */
  public ServerClientSocket(Socket socket) throws IOException {
    super(socket);
  }

  /**
   * Wenn der ServerClientSocket an den ClientCommunicator uebergeben wird kann
   * erst die Referenz eingegeben und eine Nummer fuer den Client vergeben werden.
   */
  public void setClientCommunicator(ClientCommunicator cc, byte clientNr) {
    clientCommunicator = cc;
    this.clientNr = clientNr;
  }

  /**
   * Diese Routine verschickt unterschiedlich Nachrichten und Daten an den Client.
   * @param id	  Identifikation, um zu	erkennen welche	Art von	Nachricht gesendet wird.
   * @param data  Daten, die verschickt	werden
   * @exception	IOException  Falls waehrend der	Uebertragung ein Fehler	auftritt.
   */
  public void sendData(byte id,	Object data) throws IOException{
    ObjectOutputStream outStream = getOutStream();
    outStream.writeByte(id);
    switch (id)	{
      case RAM_INFO:
        outStream.writeObject(data);
        break;
      case CORE_INFO:
        outStream.writeObject(data);
        break;
      case PARAM_INFO:
        GameSetupInfo gsi = (GameSetupInfo) data;
        gsi.selfIndex = clientNr;
        outStream.writeObject(data);
        break;
      case WAIT_FOR_CORES:
        outStream.writeByte(((Byte)data).byteValue());
        break;
      case CYCLE_INFO:
        CycleInfo cycleInfo = (CycleInfo) data;
        outStream.writeObject(cycleInfo.pc);
        outStream.writeObject(cycleInfo.writeAddr);
        outStream.writeObject(cycleInfo.writeSize);
        break;
      default:
        if (id == BYE) System.out.println("- ServerClientSocket.sendData : BYE to " + clientNr);
    }
    outStream.flush();
  }

  /**
   * Reagiert auf verschieden Nachrichten. Bei BYE und CONNECTION_ERROR
   * wird die Verbindung geschlossen und bei REAYD_FOR_DATA wird dem
   * ClietCommunicator gemeldet, das der Client bereit ist Daten zu empfangen.
   * @param msgID  Die Identifikation der Nachricht, die empfangen wurde.
   * @return  true, wenn die Nachrichtenverarbeitung fortgesetzt werden	soll.
   * @exception	IOException  wenn die Nachricht	unbekannt ist.
   */
  protected boolean handleMessage(byte msgID) throws IOException {
    switch (msgID) {
      case BYE:
      case CONNECTION_ERROR:
        clientCommunicator.killClient(this);
        (getSocket()).close();
        return false;
      case READY_FOR_DATA:
        clientCommunicator.incrementClientReady();
        return true;
      default:
        throw new IOException("Unbekannte Nachricht");
    }
  }

  /**
   * Liefert den ObjectInputStream fuer diese Netzwerkverbindung.
   * @return  Der ObjectInputStream
   */
  public ObjectInputStream getInStream() {
    return super.getInStream();
  }

}
