/*
 * @(#)ServerFrame.java    11.6.1999
 */


package corewar.server;

import java.util.Vector;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;

import corewar.CoreInfo;
import corewar.GameInfo;
import corewar.StatusDisplay;
import corewar.MessageBox;
import corewar.ToggleAction;
import corewar.CWProtocol;
import corewar.GameSetupInfo;


import java.awt.Frame;
import java.awt.Dialog;
import java.awt.Label;
import java.awt.Button;
import java.awt.GridLayout;  
import java.awt.GridBagLayout;  
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.TextField;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Rectangle;
  
import java.awt.event.WindowAdapter;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
  
import corewar.ParamInfo;

/**
 * Die Klasse ServerFrame ist die Hauptklasse des Server-Programmes. Im Konstruktor
 * stellt sie eine Dialogbox mit den notwendigen Buttons, dem Statusdisplay und der
 * Coreliste zusammen und verbindet die Buttons mit je einem Listener, der dann entweder
 * direkt oder durch Aufruf einer Methode auf den Knopfdruck reagiert.
 * @author  Carsten Buschmann
 */
public class ServerFrame extends Frame implements CWProtocol {
    
    /** Referenz auf das StatusDisplay im Server-Hauptfenster */
    private corewar.StatusDisplay statusDisplay=null;
    /** Referenz auf die Coreliste im Server-Hauptfenster */
    private ServerCoreList coreList=null;
    /** Referenz auf eine laufendes Spiel */
    private Tournament tournament=null;
    /** Refenrenz auf den Netmangager */
    private NetManager netManager=null; 
    /** Referenz auf das Parameter-Modul */
    private ParamModule params=null;
    
    /** Referenzen auf die im Server-Hauptfenster befindlichen Buttons */
    private Button paramButton;    
    private Button connectButton;    
    private Button disconnectButton;    
    private Button directButton;    
    private Button startButton;
    private Button stopButton;
    private Button deleteButton;
     
    /**
     * Erzeugt einen neuen ServerFrame. Initialisiert das Hauptfenster,
     * den NetManager und das Parameter-Modul
     */
    public ServerFrame() {
          // Dialogbox initialisieren und Layout festsetzen
          super( "Server" );
          setBackground(Color.lightGray);
          GridBagLayout grid = new GridBagLayout();
          GridBagConstraints gc = new GridBagConstraints();
          setLayout( grid );
          
          //NetManager instanzieren
          netManager = new NetManager( this ); 
          //ParamModule instanzieren
          params = new ParamModule( this );
          
          //Label fr das Statusdisplay instanzieren und einfgen
          Label label = new Label( "Status:" );
          gc.insets = new Insets( 5, 5, 5, 5 );
          gc.gridx = 0;
          gc.gridy = 0;
          gc.anchor = GridBagConstraints.WEST;
          grid.setConstraints( label, gc );
          add( label );
          
          //Statusdisplay instanzieren und einfgen, Anz. der akt. Cores ausblenden
          statusDisplay = new StatusDisplay();
          statusDisplay.setParamInfo( params.getParamInfo() );
          gc.gridy = 1;
          gc.gridheight = 5;
          grid.setConstraints( statusDisplay, gc );
          add(statusDisplay);
          statusDisplay.setVisible( statusDisplay.INFO_ACTIVE_CORES, false);
          
          //Label fr die Coreliste instanzieren und einfgen
          label = new Label( "Coreliste:" );
          gc.gridy = 6;
          gc.gridheight = 1;
          grid.setConstraints( label, gc );
          add( label );

          //Coreliste instanzieren und einfgen
          coreList = new ServerCoreList(10);
          gc.gridy = 7;
          gc.gridheight = 4;
	        gc.fill = GridBagConstraints.HORIZONTAL;
          grid.setConstraints( coreList, gc );
          add(coreList);
          
          //Beenden-Button instanzieren, mit Listner verbinden und einfgen
          Button button = new Button( "Beenden" );
          button.addActionListener(new ExitListener());
          gc.gridy = 11;
          gc.gridheight = 1;
          gc.fill = 0;
          grid.setConstraints( button, gc );
          add( button );
          
          //---------------------------------------------------------
          
          //Parameter-Button instanzieren, mit Listner verbinden und einfgen
          paramButton = new Button( "Parameter" );
          paramButton.addActionListener(new ParamListener());
          gc.fill = GridBagConstraints.HORIZONTAL;
          gc.gridx = 1;
          gc.gridy = 1;
          grid.setConstraints( paramButton, gc );
          add( paramButton );
          
          
          //Anmelden-Button instanzieren, mit Listner verbinden und einfgen
          connectButton = new Button( "Anmelden" );
          ConnectListener c = new ConnectListener(this);
          c.serverFrame = this;
          connectButton.addActionListener(c);
          gc.gridy = 3;
          grid.setConstraints( connectButton, gc );
          add( connectButton );
          
          //Abmelden-Button instanzieren, mit Listner verbinden und einfgen
          disconnectButton = new Button( "Abmelden" );
          disconnectButton.addActionListener(new DisconnectListener());
          disconnectButton.setEnabled( false );
          gc.gridy = 4;
          grid.setConstraints( disconnectButton, gc );
          add( disconnectButton );
          
          //Freigeben/Sperren-Button instanzieren, mit Listner verbinden und einfgen
          directButton = new Button( "Freigeben" );
          String[] s = { "Freigeben", "Sperren" };
          DirectListener d = new DirectListener( directButton, s, false );
          gc.gridy = 5;
          grid.setConstraints( directButton, gc );
          add( directButton );
          
          //-------------------------------------------------------------
          
          //Spiel starten-Button instanzieren, mit Listner verbinden und einfgen
          startButton = new Button( "Spiel starten" );
          startButton.addActionListener(new StartListener());
          startButton.setEnabled( false );
          gc.gridy = 7;
          grid.setConstraints( startButton, gc );
          add( startButton );
          
          //Spiel abbrechen-Button instanzieren, mit Listner verbinden und einfgen
          stopButton = new Button( "Spiel abbrechen" );
          stopButton.addActionListener(new StopListener());
          stopButton.setEnabled( false );
          gc.gridy = 8;
          grid.setConstraints( stopButton, gc );
          add( stopButton );
          
          //Liste lschen-Button instanzieren, mit Listner verbinden und einfgen
          deleteButton = new Button( "Liste lschen" );
          deleteButton.addActionListener(new deleteListener());
          gc.gridy = 10;
          grid.setConstraints( deleteButton, gc );
          add( deleteButton );
          
          
          //Beim Schlieen des Fensters Programm beenden
          addWindowListener(new WindowAdapter() {
              public void windowClosing(WindowEvent event) {
                  doExit();
              }
          });
          
          setResizable( false );
          pack();
          centerScreen();
    }

    private void centerScreen() {
        Dimension dim = getToolkit().getScreenSize();
        Rectangle abounds = getBounds();
        setLocation((dim.width - abounds.width) / 2,
                    (dim.height - abounds.height) / 2);
    }

    /**
     * Fgt ein CoreInfo-Objekt in die Coreliste ein, sind genau zwei Cores in der Liste, 
     * wird der "Spiel starten" Button aktiviert
     */
    public void addCoreInfo( CoreInfo ci ) {
      	ci.alive = true;
      	coreList.addElement( ci );
      	Vector v = getCoreInfoVector();
      	if ( v.size() == 2 ) startButton.setEnabled( true );
    }

    
    /**
     * Beendet das Programm. Evtl. noch bestehende Verbindungen zu Clients oder zu einem
     * Metaserver werden beendet.
     */
    private void doExit() {
        if (netManager != null ) {
        	 netManager.closeClients();
        	 if (disconnectButton.isEnabled()) netManager.closeMetaConnect();
        }
        
        System.exit(0);
    }

    /**
     * Leert die Coreliste und weist den Netmanager an, alle Clients zu beenden. Ist sendCoresNeeded
     * true, wird darber hinaus die Nachricht CORES_NEEDED an den Metaserver geschickt
     * @param sendCoresNeede Bestimmt, ob der Metaserver ber die bentigte Coreanzahl benachrichtigt wird
     */
    private void emptyList(boolean sendCoresNeeded) {
    	  coreList.removeAllElements();
    	  netManager.closeClients();  
    	  if ( sendCoresNeeded && disconnectButton.isEnabled() ) 
    	  	sendData( CORES_NEEDED, new Byte(getParamInfo().getCoreNumber()) );
    	  startButton.setEnabled( false )
    }

    /**
     * Liefert den Vector mit den CoreInfo-Objekten aus der Coreliste zurck
     * @return Vector mit den CoreInfo-Objekten aus der Coreliste
     */
    public Vector getCoreInfoVector() {
    	  return coreList.getElements();
    }
    
    /**
     * Liefert ein ParamInfo-Objekt mit den aktuellen Paramtern zurck
     * @return  ein ParamInfo-Objekt mit den aktuellen Paramtern
     */
    public ParamInfo getParamInfo() {
      	return params.getParamInfo();
    }
    
    
    /**
     * Liefert eine Referenz auf das Statusdisplay
     * @return eine Referenz auf das Statusdisplay 
     */
    public StatusDisplay getStatusDisplay() {
      	return statusDisplay;
    }

    /**
     * Ldt den Namen des zuletzt eingegebenen Metaservers aus der Konfigurationsdatei
     * und liefert ihn zurck.
     * Dazu mssen vorher die Parameter aus der selben Datei gelesen werden, sie werden
     * jedoch nicht weiter beachtet. Tritt ein Fehler auf, wird eine Fehlermeldung 
     * ausgegeben.
     * @return den Namen des zuletzt eingegebenen Metaservers als String
     */
	  private String loadServerName() {
        String serverName = "";
        try {
          FileInputStream fis = new FileInputStream(ParamModule.fileName);
          ObjectInputStream oip = new ObjectInputStream(fis);
          ParamInfo pi = (ParamInfo)oip.readObject();
          serverName  = (String)oip.readObject();
          fis.close();
        }
        catch( Exception e ) {}//Message( "Fehler", "Parameter konnten nicht gelesen werden.", serverFrame ); }
        return serverName;
	  }
    
    /**
     * Teilt dem ServerFrame mit, da die Verbindung zu Metaserver beendet wurde. Setzt daraufhin
     * den Eintrag im Statusdisplay und den Zustand der Buttons im Hauptfenster dementsprechend
     */
    public void metaConnectionClosed() {
    	  connectButton.setEnabled( true );
    	  disconnectButton.setEnabled( false );
      	paramButton.setEnabled( true );
     	  statusDisplay.setServer( "" );
    	  directButton.setEnabled( true );
    }
    
    /**
     * Mit dieser  Methode teilt der Netmanager dem ServerFrame mit, da sich der letzte 
     * Client abgemeldet hat. Daraufhin wird ein evtl. laufendes Spiel abgebrochen
     */
    public void noMoreClients() {
       if ( tournament != null) {
         tournament.stop();
         tournament = null;
         tournamentStopped();
        }
    }

    /**
     * Speichert den Namen des Metaservers in der Konfigurationsdatei. Dazu werden zunchst die
     * Parameter aus der Datei gelesen. Dann wird die Datei zum Schreiben geffnet, die Parameter
     * und dann der Name des Metaservers hineingeschrieben. Treten Fehler auf, wir eine Fehlermeldung
     * ausgegeben.
     * @param Name des Metaservers als String
     */
	  private void saveServerName(String serverName) {
        boolean error = false;
        try {
          FileInputStream fis = new FileInputStream(ParamModule.fileName);
          ObjectInputStream oip = new ObjectInputStream(fis);
          ParamInfo pi = (ParamInfo)oip.readObject();
          fis.close();
        }
        catch( Exception e ) {error = true;}//Message( "Fehler", "Parameter konnten nicht gelesen werden.", this ); }
        if (!error){
          try {
            FileOutputStream fos = new FileOutputStream(ParamModule.fileName);
            ObjectOutputStream oop = new ObjectOutputStream(fos);
            oop.writeObject(getParamInfo());
            oop.writeObject(serverName);
            fos.close();
          }
          catch( Exception e ) {error = true;}
        }
        if (error) {
      	   MessageBox m = new MessageBox( "Servername konnte nicht gespeichert werden.", "Fehler", this, MessageBox.OK );
   	       m.setBackground(Color.lightGray);
           int i = m.execute();
        }
	  }
    
    /**
     * Verschickt Daten an den Netmanager. Bei den Nachrichten CYCLE_INFO und END_OF_ROUND
     * werden im Statusdisplay die entsprechende Werte fr Zyklus/Runde incrementiert und
     * und evtl. gestorbene Cores in der Coreliste als tot markiert. Handelte 
     * es sich um die letzte Runde, wird das Turnier durch Aufruf von TournamentStopped() als
     * beended angesehen
     * @param id ID der Nachricht
     * @param data Evtl. zur Nachricht gehrige Daten
     */
    public void sendData(byte id, Object data) {
      	if (id == END_OF_ROUND) {
	   if (statusDisplay.getGameInfo().getRoundNumber() < params.getParamInfo().getTotalRounds()) {
      	     statusDisplay.setCycle( 1 );
             statusDisplay.incrementRound();
            //alle Cores in der Liste als lebendig markieren
            Vector coreVector = getCoreInfoVector();
            for (int i = 0; i<coreVector.size(); i++) ((CoreInfo)coreVector.elementAt(i)).alive = true;
	    coreList.setElements(coreVector);
           }
        }
        try { netManager.sendData(id, data); } 
        catch( IOException e ) {
          MessageBox m = new MessageBox( "Fehler in ServerFrame.sendData(): " + e.getMessage(), "Fehler", this, MessageBox.OK );
          m.setBackground(Color.lightGray);
          int i = m.execute();
        }
      	if (id == CYCLE_INFO) {
		statusDisplay.incrementCycle();
      		// in dieser Runde gestorbene Core in der Liste als tot markieren
      		Vector coreVector = getCoreInfoVector();
      		CycleInfo ci = (CycleInfo)data;
      		boolean died = false;
      		int current = 0;      		
      		for (int i = 0; i < ci.writeSize.length; i++) { 
      			while ((current < coreVector.size()) && (!((CoreInfo)coreVector.elementAt(current)).alive)) current++;
      			if ((current < coreVector.size()) && (ci.writeSize[i] == -1)) {
      				((CoreInfo)coreVector.elementAt(current)).alive = false;
      				died = true;
      			}
                        current++;
      		}
          if (died) coreList.setElements(coreVector);
    	}
      	if (id == END_OF_ROUND) {
	   if (statusDisplay.getGameInfo().getRoundNumber() > 
                      params.getParamInfo().getTotalRounds()) {
      	    tournamentStopped();
       	  }
	}
    }
    
    /**
     * Startet ein neues Spiel mit den Cores aus der Coreliste. Dazu werden die Buttons
     * entsprechend gesetzt, eine GameSetupInfo erzeugt und an den Netmanager geschickt,
     * dann wird ein Tournament instanziert und des gestartet.
     */
    public void startTournament() {
      	stopButton.setEnabled( true );
      	startButton.setEnabled( false );
        directButton.setEnabled( false );
      	statusDisplay.setRound( (byte)1 );
       	statusDisplay.setCycle( 1 ); 
        deleteButton.setEnabled( false );
        paramButton.setEnabled( false );

        GameSetupInfo gsi = new GameSetupInfo(getParamInfo(), getCoreInfoVector(), (byte)0);
        sendData( PARAM_INFO, gsi );
    	tournament = new Tournament( this ); 
    	tournament.start(); 
    }
    

    /**
     * Bricht ein laufendes Spiel ab. Danach werden die nach dem Spielende notwendigen
     * arbeiten durch tournamentStopped() erledigt.
     */
    private void stopTournament() {
      if ( tournament != null) {
         tournament.stopTournament();
         tournamentStopped();
        }
    }


    /**
     * Erledigt die Aufrumarbeiten nach einem Spiel, d.h. setzt die Buttons im 
     * Hauptfenster entsprechend und leert mittels emptyList() die Coreliste
     */
    private void tournamentStopped() {
      	statusDisplay.setRound( (byte)0 );
      	statusDisplay.setCycle( 0 ); 
    	  stopButton.setEnabled(false);
    	  startButton.setEnabled(true);
        deleteButton.setEnabled( true );
        directButton.setEnabled( true );
        paramButton.setEnabled( true );
     	  emptyList(true);
    }        
//================== Hier beginnen die Listener ===================================


    /**
     * Listener des "Beenden" Buttons: Schliet das Programm durch Aufruf von doExit()
     */
   class ExitListener implements ActionListener {
      public void actionPerformed(ActionEvent ae) {
        doExit();
      }
   }


    /**
     * Listener des "Parameter" Buttons: Ruft eine Dialogbox zum Einstellen der Parameter
     * auf, berprft die eingestellten Werte auf Gltigkeit und setzt die Parameter neu.
     */
   class ParamListener implements ActionListener {
      public void actionPerformed(ActionEvent ae) {
      	params.paramDialog();
      }
   }
   
    /**
     * Listener des "Anmelden" Buttons: Ruft eine Dialogbox zum Eingeben des Metaservers auf,
     * und versucht, ber NetManager.ConnectToMeta() eine Verbindung zu diesem Rechner 
     * aufzubauen und sich bei ihm anzumelden.
     */
   class ConnectListener implements ActionListener {
   	  private Dialog dialog;
   	  public Frame serverFrame;
      private int result = 0;
    
      public ConnectListener( Frame server ) {
      	super();
      	serverFrame = server;
      }

      public void actionPerformed(ActionEvent ae) {
          int error = 0;
          result = 0;
      	  emptyList(false);
      
      	  // voreingestellten Servernamen von Platte laden
      	  String serverName = loadServerName(); 
          //Dialogbox instanzieren
          dialog = new Dialog( serverFrame, "Verbinden", true );
        	dialog.setLayout(new GridLayout(2,2));
        	dialog.add( new Label( "Verteiler:"));
        	dialog.setResizable( false );
        	
        	//Textfeld instanzieren und einfgen
        	TextField text = new TextField(serverName, 20 );
        	dialog.add(text);
        	
        	//Listener zum Schlieen des Fensters
        	dialog.addWindowListener(new WindowAdapter() {
              public void windowClosing(WindowEvent event) {
                  dialog.setVisible( false );
              }
          });

          //Listener des OK-Buttons
          ActionListener okListener = new ActionListener() {
              public void actionPerformed(ActionEvent event) {
                result = 1;
                dialog.setVisible( false );
              }
          };
        	
        	//Ok-Button instanzieren, mit Listener verbinden und einfgen
        	Button okButton = new Button("Verbinden");
        	okButton.addActionListener( okListener );
        	dialog.add(okButton);
   
          //Listener des Abbrechen-Buttons
          ActionListener cancelListener = new ActionListener() {
              public void actionPerformed(ActionEvent event) {
                  dialog.setVisible( false );
              }
          };     	
        	//Abbrechen-Button instanzieren, mit Listener verbinden und einfgen
        	Button cancelButton = new Button("Abbrechen");
        	cancelButton.addActionListener( cancelListener );
        	dialog.add(cancelButton);
        	
					dialog.pack();
					dialog.setVisible( true );        
        
          InetAddress addr = null;
          serverName = text.getText();
          saveServerName( serverName );
          if( result == 1 ) {
            	//Adresse des Metaservers ermitteln
              try {
            	    addr = InetAddress.getByName( serverName );
              }
              catch (UnknownHostException uhe) {
            	   MessageBox m = new MessageBox( "Server nicht gefunden: " + serverName, "Fehler", serverFrame, MessageBox.OK );
        	       m.setBackground(Color.lightGray);
                 int i = m.execute();
                 error = 1;
              }
              catch( Exception e ){
            	   MessageBox m = new MessageBox( "Fehler beim verbinden mit " + serverName + ": " + e.getMessage(), "Fehler", serverFrame, MessageBox.OK );
        	       m.setBackground(Color.lightGray);
                 int i = m.execute();
                 error = 2;
              }
            	//Mit Metaserver verbinden
            	if (error == 0) {
            	  try {
              	  netManager.connectToMeta( addr );
                	paramButton.setEnabled( false );
              	  connectButton.setEnabled( false );
              	  disconnectButton.setEnabled( true );
      	          statusDisplay.setServer( serverName );
                  directButton.setEnabled( false );
              	}
              	catch(IOException ioe) {
            	     MessageBox m = new MessageBox( "Beim Verbinden mit " + serverName +" ist ein Fehler aufgetreten: " + ioe.getMessage(), "Fehler", serverFrame, MessageBox.OK );
        	         int i = m.execute();
            	  }
                catch( Exception e ){
              	   MessageBox m = new MessageBox( "Fehler beim verbinden mit " + serverName + ": " + e.getMessage(), "Fehler", serverFrame, MessageBox.OK );
          	       m.setBackground(Color.lightGray);
                   int i = m.execute();
                }
            	}
          }
      }
   }


    /**
     * Listener des "Abmelden" Buttons: bricht eine evtl. Verbindung zu einem Metaserver ab.
     */
   class DisconnectListener implements ActionListener {
      public void actionPerformed(ActionEvent ae) {
      	  netManager.closeMetaConnect();
      	  metaConnectionClosed();
      }
   }

    /**
     * Listener des "Freigeben/Sperren" Buttons: Freigeben/Sperren des Servers fr sich direkt
     * anmeldende Client mittels NetManager.seIndipendentMode().
     */
   class DirectListener extends ToggleAction {
      public DirectListener(Button button, String[] labels, boolean activated) {
         	super( button, labels, activated );
      }
      
      public void buttonToggled(boolean activated) {
      	  connectButton.setEnabled( !activated );
      	  netManager.setIndependentMode( activated ); 
      }
   }

    /**
     * Listener des "Spiel starten" Buttons: Startet das Spiel mittels startTournament().
     */
   class StartListener implements ActionListener {
      public void actionPerformed(ActionEvent ae) {
      	  startTournament();
      	
      }
   }

    /**
     * Listener des "Spiel abbrechen" Buttons: Bricht ein laufendes Spiel mittels stopTournament()
     * ab
     */
   class StopListener implements ActionListener {
      public void actionPerformed(ActionEvent ae) {
 	 stopTournament();
      }
   }

    /**
     * Listener des "Liste lschen" Buttons: Lscht die Liste mittels emptyList();
     */
   class deleteListener implements ActionListener {
      public void actionPerformed(ActionEvent ae) {
      	  emptyList(true);
      }
   }

    /**
     * Main-Routine des Programmes: ffnet das Hauptfenster des Servers
     */

      public static void main(String[] args) {
          ServerFrame sf = new ServerFrame();
          sf.setVisible(true);
      }
}