/*
 * @(#)ServerMetaSocket.java	10.6.1999
 */

package	corewar.server;

import corewar.NetworkSocket;
import corewar.server.ListWatcher;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.net.InetAddress;
import java.net.Socket;

/**
 * Die Klasse MetaServerSocket regelt die Kommunikation	mit dem Metaserver.
 *
 * @author  Stephan Keitz
 */
public class ServerMetaSocket extends NetworkSocket {

  /** Referenz auf die Klasse NetManager, um Nachrichten durchzureichen. */
  private NetManager netManager;
  /** Die Netzwerkidentifikation fuer den Server. */
  private static final byte[] i_am_server = { I_AM_SERVER };

  /**
   * Der Konstruktor stellt eine Verbindung zum	Metaserver her und sendet
   * die Parameter fuer	ein neues Spiel.
   * @param nm	      Eine Referenz zum NetManager
   * @param address  IP	Adresse	des MetaServers
   * @exception	IOException  Falls waehrend des	Verbindungsaufbaus ein Fehler auftritt.
   */
  public ServerMetaSocket( NetManager nm, InetAddress address) throws IOException {
    super(address, DEFAULT_META_SERVER_PORT, i_am_server);
    netManager = nm;
    sendData(PARAM_INFO, netManager.getParamInfo());
  }

  /**
   * Diese Routine verschickt unterschiedlich Nachrichten und Daten an den Metaserver.
   * Dazu gehoeren die Parameter, die eigene Portnummer und die Anzahl der noch
   * benoetigten Clients
   * @param id    Identifikation, um zu erkennen welche	Art von Nachricht gesendet wird.
   * @param data  Daten, die verschickt werden
   * @exception IOException  Falls waehrend der	Uebertragung ein Fehler auftritt.
   */
  public void sendData(byte id,	Object data) throws IOException	{
    ObjectOutputStream outStream = getOutStream();
    outStream.writeByte(id);
    switch (id)	{
      case CORES_NEEDED:
        outStream.writeByte(((Byte)data).byteValue());
        break;
      case PARAM_INFO:
        outStream.writeObject(data);
        outStream.writeInt(netManager.getPortNumber());
        break;
      default:
    }
    outStream.flush();
  }

  /**
   * Diese Routine interpretiert ankommende Nachrichten und schliesst den Socket
   * wenn ein Fehler auftritt.
   * @param msgID  Die Identifikation der Nachricht, die empfangen wurde.
   * @return  true, wenn die Nachrichtenverarbeitung fortgesetzt werden	soll.
   * @exception	IOException  wenn die Nachricht	unbekannt ist.
   */
  protected boolean handleMessage(byte msgID) throws IOException {
    switch (msgID) {
      case BYE:
      case CONNECTION_ERROR:
        (getSocket()).close();
        netManager.metaConnectionClosed();
        return false;
      case THIS_IS_YOUR_CLIENT:
        ObjectInputStream inStream = getInStream();
        byte id = inStream.readByte();
        netManager.queueID(id);
        sendData( READY_FOR_CLIENT, null);
        return true;
      default:
        throw new IOException("Unbekannte Nachricht");
    }
  }

}