/*
 * @(#)Tournament.java   19.06.1999
 */
 
package corewar.server;

import corewar.CWProtocol;
import corewar.ParamInfo;
import java.util.Vector;
import java.lang.Class;


/**
 * Die Klasse fuehrt ein Spiel aus, indem es fuer jede Runde
 * einen neuen Simulator erzeugt.
 *
 * @author Burkhard Moser
 */
public class Tournament extends Thread implements CWProtocol
{
   /** server  Enthaelt eine Referenz auf das ServerFrame-Objekt,
    *          das dieses Tournament-Objekt erzeugt hat.*/
   private ServerFrame server;
   
   /** simulator  Enthaelt eine Referenz auf den erzeugten Simulator.*/
   private Simulator simulator;
   

   /**
    * Initialisiert das Objekt.
    * 
    * @param server  Referenz auf den das Objekt erzeugenden ServerFrame.
    */
   public Tournament(ServerFrame server)
   {
      this.server = server;
   }


   /**
    * Holt sich vom ServerFrame die Spielparameter
    * und uebergibt diese an runTournament().
    */
   public void run()
   {
      ParamInfo params = server.getParamInfo();
      runTournament(params);  
   }

   
   /**
    * Fuehrt das Spiel durch, indem es in einer Schleife je einen
    * Simulator erzeugt und diesen Runden durchfuehren laesst.
    * Liefert false zurueck, wenn das Spiel durch einen Aufruf
    * von stop() beendet wurde, ansonsten true.
    * 
    * @param params  Die fuer dieses Spiel eingestellten Spielparameter.
    * @return        Boolescher Wert, der false ist, wenn das Spiel durch
    *                einen Aufruf von stop() beendet wurde, und sonst true.
    */
   private boolean runTournament(ParamInfo params)     
   {                                                  
      boolean stop_flag = true;
      Vector coreInfoVector = getCoreInfoVector();    

      /**Die folgende Schleife fuehrt genauso viele Runden aus, wie in
       * den Spielparametern eingestellt wurde. Sie endet entweder,
       * wenn die letzte Runde ausgefuehrt wurde, oder wenn stop_flag
       * false ist, was genau dann geschieht, wenn Stop gedrueckt wurde.*/
      for (int i = 1; (i <= params.getTotalRounds()) && stop_flag; i++)
      {
         /**Erzeugen eines Simulators.*/
         simulator = new Simulator(this, coreInfoVector, params);     

         /**Start einer Runde. Der boolesche Rueckgabewert wird
          * an stop_flag uebergeben*/
         stop_flag = simulator.runRound(params);                
      }

      /**Aufloesen der Referenz auf den Simulator und somit
       * Uebergabe an den Garbage Collector.*/
      this.simulator = null;      
      
      /**stop_flag ist noch true, wenn nicht Stop gedrckt wurde, false sonst.*/
      return stop_flag;          
   }                                            


   /**
    * Stoppt den laufenden Zyklus, indem die Methode stop()
    * des Simulators aufgerufen wird.
    */
   public void stopTournament()
   {
      /**Die folgende Schleife stellt sicher, dass Scheduler
       * bereits instanziert ist, wenn stop() aufgerufen wird
      while(true)
      {
         if (Class.forName("corewar.server.Simulator") != null)
         {
            break;
         }
      }
      */
      simulator.stop();  
   }


   /**
    * Versendet Daten ueber den abgelaufenen Spielzyklus an den ServerFrame.
    * 
    * @param id    Zahl, die codiert um welche Art Information
    *              es sich bei dem folgenden Object data handelt.
    * @param data  Object, das die eigentlichen zu
    *              uebertragenden Daten enthaelt.
    */
   public void sendData(byte id, Object data)
   {
      server.sendData(id, data);  
   }


   /**
    * Holt sich vom ServerFrame einen Vector, dessen Elemente
    * CoreInfo-Objekte sind, Objekte, die u.a. die in das Ram zu
    * schreibenden Informationen der Cores enthalten.
    *
    * @return  Vector, dessen Elemente CoreInfo-Objekte sind.
    */
   private Vector getCoreInfoVector()
   {
      Vector coreInfoVector = server.getCoreInfoVector();
      return coreInfoVector;
   }
}
